import { Inject, Injectable, InjectionToken, Injector, NgModule, Pipe } from '@angular/core';
import { AlertController, DeepLinker, Events, LoadingController, Nav, NavParams, Platform, ToastController } from 'ionic-angular';
import { AUTH_SETTINGS, AUTH_TYPE, AbstractShowMessage, ApiDateTimeService, AuthBaseService, AuthHandlerType, AuthInterceptorHandlerService, BaseMessage, CriticalError, DateTimePipe, DateViewConverter, DefaultDateViewConverterService, DefaultTimeViewConverterService, HttpInterceptorBaseService, I18nBaseService, LoadingDefault, MessageDisplayerService, MessageUtils, ShowMessageLinkActionType, ShowMessageType, SqlUtilsFn, TimePipe, TimeViewConverter, UserPermissionService, UtilsFn } from 'base-foundation';
import { HTTP_INTERCEPTORS, HttpClient } from '@angular/common/http';
import { TranslateLoader, TranslateModule, TranslateService } from '@ngx-translate/core';
import { TranslateHttpLoader } from '@ngx-translate/http-loader';
import { delay, map } from 'rxjs/operators';
import { CommonModule, DatePipe } from '@angular/common';
import { StatusBar } from '@ionic-native/status-bar';
import { SplashScreen } from '@ionic-native/splash-screen';
import { Camera } from '@ionic-native/camera';
import { File } from '@ionic-native/file';
import { Geolocation } from '@ionic-native/geolocation';
import { HTTP } from '@ionic-native/http';
import { ImagePicker, OutputType } from '@ionic-native/image-picker';
import { QRScanner } from '@ionic-native/qr-scanner';
import { Subject, forkJoin, of } from 'rxjs';
import { SpeechRecognition } from '@ionic-native/speech-recognition';
import { SQLite } from '@ionic-native/sqlite';
import { Network } from '@ionic-native/network';
import { Storage } from '@ionic/storage';

var __extends = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Serviço responsável pela apresentação de mensagens
 */
var ShowMessageService = /** @class */ (function (_super) {
    __extends(ShowMessageService, _super);
    /**
     * Método construtor que inicializa a classe
     * @param toastCtrl Serviço de TOAST
     * @param alertCtrl Serviço de Alerta
     * @param i18n Serviço de internacionalização
     */
    function ShowMessageService(toastCtrl, alertCtrl) {
        var _this = _super.call(this) || this;
        _this.toastCtrl = toastCtrl;
        _this.alertCtrl = alertCtrl;
        return _this;
    }
    /**
     * Método responsável por apresentar uma mensagem
     * @param {?} showMessageOptions
     * @return {?}
     */
    ShowMessageService.prototype.displayMessage = /**
     * Método responsável por apresentar uma mensagem
     * @param {?} showMessageOptions
     * @return {?}
     */
    function (showMessageOptions) {
        switch (showMessageOptions.messageType) {
            case ShowMessageType.NotificationSuccess:
                this.showNotification(showMessageOptions);
                break;
            case ShowMessageType.DialogAlert:
            case ShowMessageType.DialogConfirmation:
                this.showDialog(showMessageOptions);
                break;
            default:
                throw new Error('Tipo de mensagem inválido');
        }
    };
    /**
     * Método responsável por apresentar uma mensagem de notificação
     * @param {?} showMessageOptions
     * @return {?}
     */
    ShowMessageService.prototype.showNotification = /**
     * Método responsável por apresentar uma mensagem de notificação
     * @param {?} showMessageOptions
     * @return {?}
     */
    function (showMessageOptions) {
        var _this = this;
        var /** @type {?} */ toast = this.toastCtrl.create({
            message: showMessageOptions.messageText,
            duration: showMessageOptions.messageLinkActionType === ShowMessageLinkActionType.No ? 3000 : 10000,
            position: 'middle',
            showCloseButton: showMessageOptions.messageLinkActionType !== ShowMessageLinkActionType.No,
            closeButtonText: showMessageOptions.messageLinkLabel
        });
        toast.onDidDismiss(function () {
            if (showMessageOptions.messageLinkActionType === ShowMessageLinkActionType.LinkNewTab) {
                _this.openLink(showMessageOptions.messageLinkAction);
            }
            else if (showMessageOptions.messageLinkActionType === ShowMessageLinkActionType.Method
                && showMessageOptions.messageLinkAction) {
                showMessageOptions.messageLinkAction();
            }
        });
        toast.present();
    };
    /**
     * Método responsável por apresentar uma mensagem de diálogo
     * @param {?} showMessageOptions
     * @return {?}
     */
    ShowMessageService.prototype.showDialog = /**
     * Método responsável por apresentar uma mensagem de diálogo
     * @param {?} showMessageOptions
     * @return {?}
     */
    function (showMessageOptions) {
        var /** @type {?} */ alert = this.alertCtrl.create({
            title: showMessageOptions.messageTitle,
            subTitle: showMessageOptions.messageText,
            message: showMessageOptions.messageDetails
        });
        if (showMessageOptions.messageCancelLabel && showMessageOptions.messageCancelLabel.trim() !== ''
            && showMessageOptions.messageType === ShowMessageType.DialogConfirmation) {
            alert.addButton({
                text: showMessageOptions.messageCancelLabel,
                handler: function () {
                    alert.dismiss(false);
                    return false;
                }
            });
        }
        if (showMessageOptions.messageConfirmLabel && showMessageOptions.messageConfirmLabel.trim() !== '') {
            alert.addButton({
                text: showMessageOptions.messageConfirmLabel,
                handler: function () {
                    alert.dismiss(true);
                    return false;
                }
            });
        }
        alert.onDidDismiss(function (data) {
            switch (data) {
                case true:
                    if (showMessageOptions.messageConfirmAction) {
                        showMessageOptions.messageConfirmAction();
                    }
                    break;
                default:
                    if (showMessageOptions.messageCancelAction) {
                        showMessageOptions.messageCancelAction();
                    }
            }
        });
        alert.present();
    };
    /**
     * Método responsável pela abertura de um link
     * @param {?} link
     * @return {?}
     */
    ShowMessageService.prototype.openLink = /**
     * Método responsável pela abertura de um link
     * @param {?} link
     * @return {?}
     */
    function (link) {
        link = link.indexOf('http://') === -1 ? "http://" + link : link;
        window.open(link, '_blank');
    };
    ShowMessageService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    ShowMessageService.ctorParameters = function () { return [
        { type: ToastController, },
        { type: AlertController, },
    ]; };
    return ShowMessageService;
}(AbstractShowMessage));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ShowMessageModule = /** @class */ (function () {
    function ShowMessageModule() {
    }
    ShowMessageModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [],
                    imports: [],
                    exports: [],
                    providers: [
                        ShowMessageService,
                        MessageDisplayerService,
                        {
                            provide: AbstractShowMessage,
                            useClass: ShowMessageService
                        }
                    ]
                },] },
    ];
    return ShowMessageModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Serviço utilizado para carregar as literais
 */
var IntlLiteralsLoaderService = /** @class */ (function () {
    /**
     * Método construtor da classe
     * @param translate
     */
    function IntlLiteralsLoaderService(translate) {
        this.translate = translate;
        /**
         * Responsável por receber as configurações do serviço de tradução para carregar as literais
         */
        this.mobileOptions = { language: 'pt-br', url: '' };
        translate.setDefaultLang(this.mobileOptions.language);
    }
    /**
     * Método responsável por setar as opções e retornar a literal atualizada
     * @param options
     */
    /**
     * Método responsável por setar as opções e retornar a literal atualizada
     * @param {?} options
     * @return {?}
     */
    IntlLiteralsLoaderService.prototype.updateOptions = /**
     * Método responsável por setar as opções e retornar a literal atualizada
     * @param {?} options
     * @return {?}
     */
    function (options) {
        try {
            Object.assign(this.mobileOptions, options);
            return this.load();
        }
        catch (/** @type {?} */ error) {
            throw new CriticalError(error);
        }
    };
    /**
     * Método responsável por setar o carregamento e retornar a literal atualizada
     */
    /**
     * Método responsável por setar o carregamento e retornar a literal atualizada
     * @return {?}
     */
    IntlLiteralsLoaderService.prototype.load = /**
     * Método responsável por setar o carregamento e retornar a literal atualizada
     * @return {?}
     */
    function () {
        return this.translate.getTranslation(this.mobileOptions.language)
            .pipe(map(function (res) { return res; }));
    };
    IntlLiteralsLoaderService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    IntlLiteralsLoaderService.ctorParameters = function () { return [
        { type: TranslateService, },
    ]; };
    return IntlLiteralsLoaderService;
}());

var __extends$1 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Serviço de internacionalização da aplicação
 */
var IntlService = /** @class */ (function (_super) {
    __extends$1(IntlService, _super);
    /**
     * Método construtor que inicializa a classe
     * @param translate Serviço de internacionalização
     */
    function IntlService(intlLiteralsLoader) {
        var _this = _super.call(this, intlLiteralsLoader) || this;
        /**
         * Atributo de opções de internacionalização
         */
        _this.options = { language: 'pt-br', url: '' };
        _this.updateOptions(_this.options).subscribe();
        return _this;
    }
    IntlService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    IntlService.ctorParameters = function () { return [
        { type: IntlLiteralsLoaderService, },
    ]; };
    return IntlService;
}(I18nBaseService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Pipe de internacionalização da aplicação
 */
var I18NPipe = /** @class */ (function () {
    /**
     * Método construtor que inicializa a classe
     * @param i18nService Serviço de internacionalização
     */
    function I18NPipe(i18nService) {
        this.i18nService = i18nService;
    }
    /**
     * Método responsável pela transformação da string
     * @param query
     * @param data
     */
    /**
     * Método responsável pela transformação da string
     * @param {?} query
     * @param {?} data
     * @return {?}
     */
    I18NPipe.prototype.transform = /**
     * Método responsável pela transformação da string
     * @param {?} query
     * @param {?} data
     * @return {?}
     */
    function (query, data) {
        if (!query || query.length === 0) {
            return '';
        }
        var /** @type {?} */ ret = this.cached(query, data);
        if (ret) {
            return ret;
        }
        return this.i18nService.format(data, query);
    };
    /**
     * Método responsável por retornar o que está em cache
     * @param {?} query
     * @param {...?} args
     * @return {?}
     */
    I18NPipe.prototype.cached = /**
     * Método responsável por retornar o que está em cache
     * @param {?} query
     * @param {...?} args
     * @return {?}
     */
    function (query) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        return '';
    };
    I18NPipe.decorators = [
        { type: Pipe, args: [{ name: 'format' },] },
    ];
    /** @nocollapse */
    I18NPipe.ctorParameters = function () { return [
        { type: I18nBaseService, },
    ]; };
    return I18NPipe;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Função responsável por indicar o diretório que contém os arquivos de tradução
 * @param {?} http
 * @return {?}
 */
function createTranslateLoader(http$$1) {
    return new TranslateHttpLoader(http$$1, './assets/i18n/', '.json');
}
var ɵ0 = (createTranslateLoader);
/**
 * Módulo responsável pela configuração da internacionalização da aplicação
 */
var IntlModule = /** @class */ (function () {
    function IntlModule() {
    }
    IntlModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        TranslateModule.forRoot({
                            loader: {
                                provide: TranslateLoader,
                                useFactory: ɵ0,
                                deps: [HttpClient]
                            }
                        })
                    ],
                    declarations: [
                        I18NPipe
                    ],
                    exports: [
                        TranslateModule,
                        I18NPipe
                    ],
                    providers: [
                        IntlService,
                        IntlLiteralsLoaderService,
                        {
                            provide: I18nBaseService,
                            useClass: IntlService
                        }
                    ]
                },] },
    ];
    return IntlModule;
}());

var __extends$2 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Serviço responsável pela apresentação de loading
 */
var ShowLoadingService = /** @class */ (function (_super) {
    __extends$2(ShowLoadingService, _super);
    /**
     * Método construtor que inicializa a classe
     * @param i18n Serviço de internacionalização
     * @param loadingController Serviço de loading
     */
    function ShowLoadingService(i18n, loadingController) {
        var _this = _super.call(this) || this;
        _this.i18n = i18n;
        _this.loadingController = loadingController;
        _this.thfContentTemplate = '<span class="thf-loading-icon"></span>{content}';
        return _this;
    }
    /**
     * @return {?}
     */
    ShowLoadingService.prototype.showLoading = /**
     * @return {?}
     */
    function () {
        if (this.loadingActived) {
            return;
        }
        else {
            var /** @type {?} */ content = '';
            if (this.loadingText) {
                content = this.loadingText.text;
            }
            this.loading = this.loadingController.create({
                content: this.thfContentTemplate.replace('{content}', content),
                spinner: 'hide'
            });
            this.loadingActived = true;
            this.loading.present();
        }
    };
    /**
     * @return {?}
     */
    ShowLoadingService.prototype.hideLoading = /**
     * @return {?}
     */
    function () {
        if (this.loadingActived) {
            this.loadingText = undefined;
            this.loadingActived = false;
            this.loading.dismiss();
        }
    };
    /**
     * @param {?} loadingText
     * @return {?}
     */
    ShowLoadingService.prototype.setLoadingText = /**
     * @param {?} loadingText
     * @return {?}
     */
    function (loadingText) {
        var /** @type {?} */ baseMessage = this.i18n.text(loadingText);
        if (this.loadingActived && this.loading) {
            this.loading.setContent(this.thfContentTemplate.replace('{content}', baseMessage.text));
        }
        this.loadingText = baseMessage;
    };
    ShowLoadingService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    ShowLoadingService.ctorParameters = function () { return [
        { type: I18nBaseService, },
        { type: LoadingController, },
    ]; };
    return ShowLoadingService;
}(LoadingDefault));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Serviço responsável por fornecer atributos e métodos que
 * poderão ser utilizados em toda a aplicação
 */
var GlobalService = /** @class */ (function () {
    /**
     * Método construtor que inicializa o serviço
     * @param i18n Serviço de internacionalização
     * @param msg Serviço de mensageria para usuário
     * @param date Serviço de formatação de data
     */
    function GlobalService(i18n, msg, loading, date) {
        this.i18n = i18n;
        this.msg = msg;
        this.loading = loading;
        this.date = date;
    }
    GlobalService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    GlobalService.ctorParameters = function () { return [
        { type: I18nBaseService, },
        { type: MessageDisplayerService, },
        { type: ShowLoadingService, },
        { type: ApiDateTimeService, },
    ]; };
    return GlobalService;
}());

var __extends$3 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Serviço utilizado para realizar o tratamento das respostas
 */
var DefaultHttpInterceptorService = /** @class */ (function (_super) {
    __extends$3(DefaultHttpInterceptorService, _super);
    /**
     * Método construtor da classe
     * @param inj
     */
    function DefaultHttpInterceptorService(inj) {
        var _this = _super.call(this) || this;
        _this.inj = inj;
        return _this;
    }
    /**
     * Método responsável por realizar o tratamento de respostas de sucesso
     * @param response
     */
    /**
     * Método responsável por realizar o tratamento de respostas de sucesso
     * @param {?} response
     * @return {?}
     */
    DefaultHttpInterceptorService.prototype.interceptResponse = /**
     * Método responsável por realizar o tratamento de respostas de sucesso
     * @param {?} response
     * @return {?}
     */
    function (response) {
        this.msg = this.inj.get(MessageDisplayerService);
        if (response.body && response.body.totvs_messages) {
            if (response.body.totvs_messages instanceof Array) {
                var /** @type {?} */ messages = response.body.totvs_messages;
                for (var _i = 0, messages_1 = messages; _i < messages_1.length; _i++) {
                    var message = messages_1[_i];
                    this.showTotvsMessage(message);
                }
            }
            else {
                this.showTotvsMessage(/** @type {?} */ (response.body.totvs_messages));
            }
        }
    };
    /**
     * Método responsável por realizar o tratamento de respostas de erro
     * @param response
     * @param request
     */
    /**
     * Método responsável por realizar o tratamento de respostas de erro
     * @param {?} response
     * @param {?} request
     * @return {?}
     */
    DefaultHttpInterceptorService.prototype.interceptError = /**
     * Método responsável por realizar o tratamento de respostas de erro
     * @param {?} response
     * @param {?} request
     * @return {?}
     */
    function (response, request) {
        var /** @type {?} */ error = /** @type {?} */ (response.error);
        if (error && error.message && !HttpInterceptorBaseService.isErrorNotificationDisabled(request)) {
            var /** @type {?} */ title = HttpInterceptorBaseService.titleFromTotvsError(error);
            var /** @type {?} */ detail = HttpInterceptorBaseService.detailFromTotvsError(error);
            this.showTotvsMessage({
                type: 'error',
                code: title,
                detail: detail
            });
        }
    };
    /**
     * Método responsável por mostrar a mensagem
     * @param message
     */
    /**
     * Método responsável por mostrar a mensagem
     * @param {?} message
     * @return {?}
     */
    DefaultHttpInterceptorService.prototype.showTotvsMessage = /**
     * Método responsável por mostrar a mensagem
     * @param {?} message
     * @return {?}
     */
    function (message) {
        var /** @type {?} */ messageType = MessageUtils.showMessageTypeFromTotvsMessage(message);
        this.msg.displayMessage({
            messageType: messageType,
            messageText: message.code,
            messageDetails: message.detail
        });
    };
    /**
     * Método responsável por realizar o tratamento da requisição/retorno
     * @param req
     * @param next
     */
    /**
     * Método responsável por realizar o tratamento da requisição/retorno
     * @param {?} req
     * @param {?} next
     * @return {?}
     */
    DefaultHttpInterceptorService.prototype.intercept = /**
     * Método responsável por realizar o tratamento da requisição/retorno
     * @param {?} req
     * @param {?} next
     * @return {?}
     */
    function (req, next) {
        return _super.prototype.intercept.call(this, req, next);
    };
    DefaultHttpInterceptorService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    DefaultHttpInterceptorService.ctorParameters = function () { return [
        { type: Injector, },
    ]; };
    return DefaultHttpInterceptorService;
}(HttpInterceptorBaseService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var InterceptorModule = /** @class */ (function () {
    function InterceptorModule() {
    }
    InterceptorModule.decorators = [
        { type: NgModule, args: [{
                    imports: [],
                    exports: [],
                    providers: [
                        DefaultHttpInterceptorService,
                        {
                            provide: HTTP_INTERCEPTORS,
                            useClass: DefaultHttpInterceptorService,
                            multi: true
                        }
                    ]
                },] },
    ];
    return InterceptorModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var DateTimeModule = /** @class */ (function () {
    function DateTimeModule() {
    }
    DateTimeModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [
                        DateTimePipe,
                        TimePipe
                    ],
                    imports: [
                        CommonModule
                    ],
                    exports: [
                        DateTimePipe,
                        TimePipe
                    ],
                    providers: [
                        {
                            provide: DateViewConverter,
                            useClass: DefaultDateViewConverterService
                        },
                        {
                            provide: TimeViewConverter,
                            useClass: DefaultTimeViewConverterService
                        },
                        DatePipe,
                        DefaultDateViewConverterService,
                        DefaultTimeViewConverterService,
                        ApiDateTimeService
                    ]
                },] },
    ];
    return DateTimeModule;
}());

var __extends$4 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __awaiter = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (undefined && undefined.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var AuthService = /** @class */ (function (_super) {
    __extends$4(AuthService, _super);
    /**
     * Método construtor da classe
     * @param authSettings Token de injeção com as configurações de autenticação
     * @param authType Token de injeção com o tipo de autenticação
     */
    function AuthService(authSettings, authType) {
        var _this = _super.call(this, authSettings) || this;
        _this.authType = authType;
        return _this;
    }
    /**
     * Método responsável executar o processo de login/autenticação
     * @return {?}
     */
    AuthService.prototype.loginFlow = /**
     * Método responsável executar o processo de login/autenticação
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this.authType.type === AuthHandlerType.oauth2)) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.manager.signinPopup()];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Método responsável executar o processo de logout
     * @return {?}
     */
    AuthService.prototype.logoutFlow = /**
     * Método responsável executar o processo de logout
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this.authType.type === AuthHandlerType.oauth2)) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.manager.signoutPopup()];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    AuthService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    AuthService.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Inject, args: [AUTH_SETTINGS,] },] },
        { type: undefined, decorators: [{ type: Inject, args: [AUTH_TYPE,] },] },
    ]; };
    return AuthService;
}(AuthBaseService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ShowLoadingModule = /** @class */ (function () {
    function ShowLoadingModule() {
    }
    ShowLoadingModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [],
                    imports: [],
                    exports: [],
                    providers: [
                        LoadingController,
                        ShowLoadingService
                    ]
                },] },
    ];
    return ShowLoadingModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var FndModule = /** @class */ (function () {
    function FndModule() {
    }
    /**
     * @return {?}
     */
    FndModule.forRoot = /**
     * @return {?}
     */
    function () {
        return {
            ngModule: FndModule,
            providers: [
                GlobalService,
                AuthService,
                {
                    provide: AuthBaseService,
                    useClass: AuthService
                }
            ]
        };
    };
    FndModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        IntlModule,
                        ShowMessageModule,
                        InterceptorModule,
                        DateTimeModule,
                        ShowLoadingModule
                    ],
                    declarations: [],
                    exports: [
                        IntlModule,
                        ShowMessageModule,
                        InterceptorModule,
                        DateTimeModule,
                        ShowLoadingModule
                    ],
                },] },
    ];
    return FndModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Esta classe simula a classe Platform e é utilizada
 * para os testes unitários automatizados
 * ```typescript
 * // Utilizar como provider
 * providers: [
 *   { provide: Platform, useClass: PlatformMock},
 *   ...
 * ]
 * ```
 */
var PlatformMock = /** @class */ (function () {
    function PlatformMock() {
    }
    /**
     * Método que simula a ação de preparado da plataforma
     * @return {?}
     */
    PlatformMock.prototype.ready = /**
     * Método que simula a ação de preparado da plataforma
     * @return {?}
     */
    function () {
        return new Promise(function (resolve) {
            resolve('READY');
        });
    };
    /**
     * Método que simula o retorno dos parâmetros de query de URL
     * @return {?}
     */
    PlatformMock.prototype.getQueryParam = /**
     * Método que simula o retorno dos parâmetros de query de URL
     * @return {?}
     */
    function () {
        return true;
    };
    /**
     * Método que simula o registro da ação para o botão 'Voltar'
     * @param {?} fn
     * @param {?=} priority
     * @return {?}
     */
    PlatformMock.prototype.registerBackButtonAction = /**
     * Método que simula o registro da ação para o botão 'Voltar'
     * @param {?} fn
     * @param {?=} priority
     * @return {?}
     */
    function (fn, priority) {
        return (function () { return true; });
    };
    /**
     * Método que simula a verificação de foco
     * @param {?} ele
     * @return {?}
     */
    PlatformMock.prototype.hasFocus = /**
     * Método que simula a verificação de foco
     * @param {?} ele
     * @return {?}
     */
    function (ele) {
        return true;
    };
    /**
     * Método que simula o retorno do 'document'
     * @return {?}
     */
    PlatformMock.prototype.doc = /**
     * Método que simula o retorno do 'document'
     * @return {?}
     */
    function () {
        return document;
    };
    /**
     * Método que simula o retorno do 'is'
     * @return {?}
     */
    PlatformMock.prototype.is = /**
     * Método que simula o retorno do 'is'
     * @return {?}
     */
    function () {
        return true;
    };
    /**
     * Método que simula o retorno do estilo de um elemento
     * @param {?} container
     * @return {?}
     */
    PlatformMock.prototype.getElementComputedStyle = /**
     * Método que simula o retorno do estilo de um elemento
     * @param {?} container
     * @return {?}
     */
    function (container) {
        return {
            paddingLeft: '10',
            paddingTop: '10',
            paddingRight: '10',
            paddingBottom: '10',
        };
    };
    /**
     * Método que simula o evento de redimensionamento de tela
     * @param {?} callback
     * @return {?}
     */
    PlatformMock.prototype.onResize = /**
     * Método que simula o evento de redimensionamento de tela
     * @param {?} callback
     * @return {?}
     */
    function (callback) {
        return callback;
    };
    /**
     * Método que simula o registro de evento de escuta
     * @param {?} ele
     * @param {?} eventName
     * @param {?} callback
     * @return {?}
     */
    PlatformMock.prototype.registerListener = /**
     * Método que simula o registro de evento de escuta
     * @param {?} ele
     * @param {?} eventName
     * @param {?} callback
     * @return {?}
     */
    function (ele, eventName, callback) {
        return (function () { return true; });
    };
    /**
     * Método que simula o retorno do método 'win'
     * @return {?}
     */
    PlatformMock.prototype.win = /**
     * Método que simula o retorno do método 'win'
     * @return {?}
     */
    function () {
        return window;
    };
    /**
     * Método que simula o retorno do método 'raf'
     * @param {?} callback
     * @return {?}
     */
    PlatformMock.prototype.raf = /**
     * Método que simula o retorno do método 'raf'
     * @param {?} callback
     * @return {?}
     */
    function (callback) {
        return 1;
    };
    /**
     * Método que simula a definição de tempo de espera
     * @param {?} callback
     * @param {?} timer
     * @return {?}
     */
    PlatformMock.prototype.timeout = /**
     * Método que simula a definição de tempo de espera
     * @param {?} callback
     * @param {?} timer
     * @return {?}
     */
    function (callback, timer) {
        return setTimeout(callback, timer);
    };
    /**
     * Método que simula o cancelamento do tempo de espera
     * @param {?} id
     * @return {?}
     */
    PlatformMock.prototype.cancelTimeout = /**
     * Método que simula o cancelamento do tempo de espera
     * @param {?} id
     * @return {?}
     */
    function (id) {
        // do nothing
    };
    /**
     * Método que simula o retorno do elemento ativo
     * @return {?}
     */
    PlatformMock.prototype.getActiveElement = /**
     * Método que simula o retorno do elemento ativo
     * @return {?}
     */
    function () {
        return document['activeElement'];
    };
    return PlatformMock;
}());

var __extends$5 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Esta classe simula a classe StatusBar e é utilizada
 * para os testes unitários automatizados
 * ```typescript
 * // Utilizar como provider
 * providers: [
 *   { provide: StatusBar, useClass: StatusBarMock},
 *   ...
 * ]
 * ```
 */
var StatusBarMock = /** @class */ (function (_super) {
    __extends$5(StatusBarMock, _super);
    function StatusBarMock() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * Método que simula o retorno do estilo padrão
     */
    /**
     * Método que simula o retorno do estilo padrão
     * @return {?}
     */
    StatusBarMock.prototype.styleDefault = /**
     * Método que simula o retorno do estilo padrão
     * @return {?}
     */
    function () {
        return;
    };
    return StatusBarMock;
}(StatusBar));

var __extends$6 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Esta classe simula a classe SplashScreen e é utilizada
 * para os testes unitários automatizados
 * ```typescript
 * // Utilizar como provider
 * providers: [
 *   { provide: SplashScreen, useClass: SplashScreenMock},
 *   ...
 * ]
 * ```
 */
var SplashScreenMock = /** @class */ (function (_super) {
    __extends$6(SplashScreenMock, _super);
    function SplashScreenMock() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * Método que simula o evento para esconder o 'splash'
     */
    /**
     * Método que simula o evento para esconder o 'splash'
     * @return {?}
     */
    SplashScreenMock.prototype.hide = /**
     * Método que simula o evento para esconder o 'splash'
     * @return {?}
     */
    function () {
        return;
    };
    return SplashScreenMock;
}(SplashScreen));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Esta classe simula a classe DeepLinker e é utilizada
 * para os testes unitários automatizados
 * ```typescript
 * // Utilizar como provider
 * providers: [
 *   { provide: DeepLinker, useClass: DeepLinkerMock},
 *   ...
 * ]
 * ```
 */
var DeepLinkerMock = /** @class */ (function () {
    function DeepLinkerMock() {
    }
    return DeepLinkerMock;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Esta classe simula a classe NavParams e é utilizada
 * para os testes unitários automatizados
 * ```typescript
 * // Utilizar como provider
 * providers: [
 *   { provide: NavParams, useClass: NavParamsMock},
 *   ...
 * ]
 * ```
 */
var NavParamsMock = /** @class */ (function () {
    function NavParamsMock() {
    }
    /**
     * Método que simula a definição de parâmetros
     * @param value
     */
    /**
     * Método que simula a definição de parâmetros
     * @param {?} value
     * @return {?}
     */
    NavParamsMock.setParams = /**
     * Método que simula a definição de parâmetros
     * @param {?} value
     * @return {?}
     */
    function (value) {
        NavParamsMock.returnParam = value;
    };
    /**
     * Método que simula o retorno do valor de uma chave
     * @param {?} key
     * @return {?}
     */
    NavParamsMock.prototype.get = /**
     * Método que simula o retorno do valor de uma chave
     * @param {?} key
     * @return {?}
     */
    function (key) {
        if (NavParamsMock.returnParam) {
            return NavParamsMock.returnParam;
        }
        return 'default';
    };
    /**
     * Parâmetro de retorno
     */
    NavParamsMock.returnParam = null;
    return NavParamsMock;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Esta classe simula a classe Nav e é utilizada
 * para os testes unitários automatizados
 * ```typescript
 * // Utilizar como provider
 * providers: [
 *   { provide: Nav, useClass: NavMock},
 *   ...
 * ]
 * ```
 */
var NavMock = /** @class */ (function () {
    function NavMock() {
    }
    /**
     * Método que simula o retorno de uma página
     * @return {?}
     */
    NavMock.prototype.pop = /**
     * Método que simula o retorno de uma página
     * @return {?}
     */
    function () {
        return new Promise(function (resolve) {
            resolve();
        });
    };
    /**
     * Método que simula a navegação de uma página
     * @return {?}
     */
    NavMock.prototype.push = /**
     * Método que simula a navegação de uma página
     * @return {?}
     */
    function () {
        return new Promise(function (resolve) {
            resolve();
        });
    };
    /**
     * Método que simula o retorno da página ativa
     * @return {?}
     */
    NavMock.prototype.getActive = /**
     * Método que simula o retorno da página ativa
     * @return {?}
     */
    function () {
        return {
            instance: {
                model: 'something',
            },
        };
    };
    /**
     * Método que simula a definição da página base
     * @return {?}
     */
    NavMock.prototype.setRoot = /**
     * Método que simula a definição da página base
     * @return {?}
     */
    function () {
        return true;
    };
    /**
     * Método que simula o registro de um Nav
     * @param {?} nav
     * @return {?}
     */
    NavMock.prototype.registerChildNav = /**
     * Método que simula o registro de um Nav
     * @param {?} nav
     * @return {?}
     */
    function (nav) {
        return;
    };
    return NavMock;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Módulo que contém serviços simulados necessários para o desenvolvimento
 * de testes unitários automatizados utilizando ionic
 */
var MockModule = /** @class */ (function () {
    function MockModule() {
    }
    MockModule.decorators = [
        { type: NgModule, args: [{
                    imports: [],
                    exports: [],
                    providers: [
                        { provide: DeepLinker, useClass: DeepLinkerMock },
                        { provide: NavParams, useClass: NavParamsMock },
                        { provide: Nav, useClass: NavMock },
                        { provide: Platform, useClass: PlatformMock },
                        { provide: SplashScreen, useClass: SplashScreenMock },
                        { provide: StatusBar, useClass: StatusBarMock }
                    ]
                },] },
    ];
    return MockModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Módulo que exporta e ativa as dependências para utilizar autenticação com fluxo oauth2
 * Para garantir que os serviços tenham uma única instância, usar AuthModule.forRoot() ao importar
 * @export
 */
var AuthModule = /** @class */ (function () {
    function AuthModule() {
    }
    /**
     * @return {?}
     */
    AuthModule.forRoot = /**
     * @return {?}
     */
    function () {
        return {
            ngModule: AuthModule,
            providers: [
                AuthService,
                UserPermissionService,
                {
                    provide: AuthBaseService,
                    useClass: AuthService
                }
            ]
        };
    };
    AuthModule.decorators = [
        { type: NgModule, args: [{
                    providers: [
                        {
                            provide: HTTP_INTERCEPTORS,
                            useClass: AuthInterceptorHandlerService,
                            multi: true
                        }
                    ]
                },] },
    ];
    return AuthModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Classe que contém os eventos que o utilitário assina e publica
 */
var CameraUtilEvents = /** @class */ (function () {
    function CameraUtilEvents() {
    }
    /**
     * Evento publicado pelo utilitário quando uma foto é tirada e o arquivo é gerado
     * no cache do aplicativo
     */
    CameraUtilEvents.CACHE_FILE_CREATED = 'CacheFile:Created';
    return CameraUtilEvents;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de camera
 * com métodos que facilitam a solicitação de permissão e apresentação da
 * camera
 */
var CameraUtilProvider = /** @class */ (function () {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * @param platform Serviço relacionado a plataforma
     * @param camera Serviço do plugin nativo de camera
     * @param global Serviço global da aplicação
     */
    function CameraUtilProvider(platform, camera$$1, global, events) {
        var _this = this;
        this.camera = camera$$1;
        this.global = global;
        this.events = events;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then(function () {
            _this.isCordova = platform.is('cordova');
            if (_this.isCordova) {
                _this.defaultOptions = {
                    quality: 75,
                    destinationType: _this.camera.DestinationType.FILE_URI,
                    encodingType: _this.camera.EncodingType.JPEG,
                    mediaType: _this.camera.MediaType.PICTURE,
                    targetWidth: 1920,
                    targetHeight: 1080,
                    correctOrientation: true
                };
            }
        });
    }
    /**
     * Método responsável por apresentar a camera para que o usuário
     * possa tirar uma foto
     * Este método publica o evento CameraUtilEvents.CACHE_FILE_CREATED quando um arquivo é criado em
     * cache (Mais informações sobre eventos {@link https://ionicframework.com/docs/api/util/Events/|Ionic Events})
     * @param {CameraOptions} options Opções da camera
     * @returns Promessa com o FILE_URI (opção padrão) ou o que for definido na opção 'destinationType' que pode ser passada como parâmetro
     */
    /**
     * Método responsável por apresentar a camera para que o usuário
     * possa tirar uma foto
     * Este método publica o evento CameraUtilEvents.CACHE_FILE_CREATED quando um arquivo é criado em
     * cache (Mais informações sobre eventos {\@link https://ionicframework.com/docs/api/util/Events/|Ionic Events})
     * @param {?=} options
     * @return {?} Promessa com o FILE_URI (opção padrão) ou o que for definido na opção 'destinationType' que pode ser passada como parâmetro
     */
    CameraUtilProvider.prototype.takePicture = /**
     * Método responsável por apresentar a camera para que o usuário
     * possa tirar uma foto
     * Este método publica o evento CameraUtilEvents.CACHE_FILE_CREATED quando um arquivo é criado em
     * cache (Mais informações sobre eventos {\@link https://ionicframework.com/docs/api/util/Events/|Ionic Events})
     * @param {?=} options
     * @return {?} Promessa com o FILE_URI (opção padrão) ou o que for definido na opção 'destinationType' que pode ser passada como parâmetro
     */
    function (options) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (_this.isCordova === false) {
                _this.global.msg.showMessage(new BaseMessage({
                    literal: 'recursoNaoEstaDisponivelMsg'
                }));
                reject();
            }
            var /** @type {?} */ cameraOptions = Object.assign({}, _this.defaultOptions);
            if (options) {
                for (var _i = 0, _a = Object.keys(options); _i < _a.length; _i++) {
                    var prop = _a[_i];
                    cameraOptions[prop] = options[prop];
                }
            }
            _this.camera.getPicture(cameraOptions)
                .then(function (imageData) {
                _this.events.publish(CameraUtilEvents.CACHE_FILE_CREATED, imageData);
                resolve(imageData);
            }, function (error) { return reject(error); }).catch(function (error) {
                var /** @type {?} */ literal;
                if (error && error === 20) {
                    literal = 'permissaoNaoConcedidaParaORecursoMsg';
                }
                else {
                    literal = 'recursoNaoEstaDisponivelMsg';
                }
                _this.global.msg.showMessage(new BaseMessage({
                    literal: literal
                }));
                reject(error);
            });
        });
    };
    /**
     * Método responsável por limpar as imagens em cache geradas durante
     * a utilização do plugin enquanto estiver ativo
     */
    /**
     * Método responsável por limpar as imagens em cache geradas durante
     * a utilização do plugin enquanto estiver ativo
     * @return {?}
     */
    CameraUtilProvider.prototype.cleanup = /**
     * Método responsável por limpar as imagens em cache geradas durante
     * a utilização do plugin enquanto estiver ativo
     * @return {?}
     */
    function () {
        if (this.isCordova) {
            this.camera.cleanup()
                .then(function () { })
                .catch(function () { });
        }
    };
    CameraUtilProvider.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    CameraUtilProvider.ctorParameters = function () { return [
        { type: Platform, },
        { type: Camera, },
        { type: GlobalService, },
        { type: Events, },
    ]; };
    return CameraUtilProvider;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de camera
 */
var CameraUtilModule = /** @class */ (function () {
    function CameraUtilModule() {
    }
    CameraUtilModule.decorators = [
        { type: NgModule, args: [{
                    providers: [
                        Camera,
                        CameraUtilProvider
                    ]
                },] },
    ];
    return CameraUtilModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

var __awaiter$1 = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator$1 = (undefined && undefined.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de arquivo
 * com métodos que facilitam a manipulação de arquivos no aplicativo
 */
var FileUtilProvider = /** @class */ (function () {
    /**
     * @ignore
     * Método construtor responsável por inicializar o serviço
     * @param platform Serviço relacionado a plataforma
     * @param file Serviço do plugin nativo de arquivo
     * @param events Serviço de eventos do Ionic
     */
    function FileUtilProvider(platform, file$$1) {
        var _this = this;
        this.file = file$$1;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then(function () { return _this.isCordova = platform.is('cordova'); });
    }
    /**
     * Método responsável por remover um arquivo
     * @param {?} uri URI do arquivo que deve ser removido
     * @return {?} Objeto que representa o resultado da remoção do arquivo
     */
    FileUtilProvider.prototype.removeFile = /**
     * Método responsável por remover um arquivo
     * @param {?} uri URI do arquivo que deve ser removido
     * @return {?} Objeto que representa o resultado da remoção do arquivo
     */
    function (uri) {
        return __awaiter$1(this, void 0, void 0, function () {
            var fileName, path, removeResult, error_1;
            return __generator$1(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.isCordova === false) {
                            throw new CriticalError('Platform is not cordova (removeFile)');
                        }
                        if (!uri || uri === '') {
                            throw new CriticalError('URI invalid (removeFile)');
                        }
                        fileName = this.getFilenameFromUri(uri);
                        path = this.getPathFromUri(uri);
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this.file.removeFile(path, fileName)];
                    case 2:
                        removeResult = _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _a.sent();
                        throw new CriticalError('Error in remove file', error_1);
                    case 4: return [2 /*return*/, removeResult];
                }
            });
        });
    };
    /**
     * Método responsável por transferir os arquivos para o storage do aplicativo
     * @param {...?} uris URIs dos arquivos que devem ser copiados
     * @return {?} Lista de URIs dos arquivos copiados para o storage do aplicativo
     */
    FileUtilProvider.prototype.fileCacheToAppStorage = /**
     * Método responsável por transferir os arquivos para o storage do aplicativo
     * @param {...?} uris URIs dos arquivos que devem ser copiados
     * @return {?} Lista de URIs dos arquivos copiados para o storage do aplicativo
     */
    function () {
        var uris = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            uris[_i] = arguments[_i];
        }
        return __awaiter$1(this, void 0, void 0, function () {
            var _this = this;
            var promisesCopy, filesCopied, error_2, promisesRemove, filesUrl, error_3;
            return __generator$1(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.isCordova === false) {
                            throw new CriticalError('Platform is not cordova');
                        }
                        promisesCopy = [];
                        uris.forEach(function (uri) {
                            var /** @type {?} */ fileName = decodeURI(_this.getFilenameFromUri(uri));
                            var /** @type {?} */ path = _this.getPathFromUri(uri);
                            promisesCopy.push(_this.file.copyFile(path, fileName, _this.file.dataDirectory, fileName)
                                .then(function (entry) {
                                return {
                                    sourcePath: path,
                                    sourceFileName: fileName,
                                    targetFile: entry
                                };
                            }));
                        });
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, Promise.all(promisesCopy)];
                    case 2:
                        filesCopied = _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        error_2 = _a.sent();
                        throw new CriticalError("Error in copy file", error_2);
                    case 4:
                        promisesRemove = [];
                        filesUrl = [];
                        filesCopied.forEach(function (fileCopied) {
                            filesUrl.push(fileCopied.targetFile.toURL());
                            promisesRemove.push(_this.file.removeFile(fileCopied.sourcePath, fileCopied.sourceFileName)
                                .then(function (result) {
                                return result;
                            }));
                        });
                        _a.label = 5;
                    case 5:
                        _a.trys.push([5, 7, , 8]);
                        return [4 /*yield*/, Promise.all(promisesRemove)];
                    case 6:
                        _a.sent();
                        return [3 /*break*/, 8];
                    case 7:
                        error_3 = _a.sent();
                        throw new CriticalError("Error in remove file", error_3);
                    case 8: return [2 /*return*/, filesUrl];
                }
            });
        });
    };
    /**
     * @ignore
     * Método responsável por recuperar o nome do arquivo de um URI
     * @param {?} uri URI do arquivo
     * @return {?}
     */
    FileUtilProvider.prototype.getFilenameFromUri = /**
     * @ignore
     * Método responsável por recuperar o nome do arquivo de um URI
     * @param {?} uri URI do arquivo
     * @return {?}
     */
    function (uri) {
        var /** @type {?} */ fileName;
        try {
            var /** @type {?} */ fileNameArray = uri.split('/');
            fileName = fileNameArray[fileNameArray.length - 1];
        }
        catch (/** @type {?} */ error) {
            throw new CriticalError('Error (getFilenameFromUri)', error);
        }
        return fileName;
    };
    /**
     * @ignore
     * Método responsável por recuperar o path do arquivo de um URI
     * @param {?} uri URI do arquivo
     * @return {?}
     */
    FileUtilProvider.prototype.getPathFromUri = /**
     * @ignore
     * Método responsável por recuperar o path do arquivo de um URI
     * @param {?} uri URI do arquivo
     * @return {?}
     */
    function (uri) {
        var /** @type {?} */ filePath;
        try {
            var /** @type {?} */ fileNameArray = uri.split('/');
            fileNameArray.splice(fileNameArray.length - 1, 1);
            filePath = fileNameArray.join('/');
        }
        catch (/** @type {?} */ error) {
            throw new CriticalError('Error (getPathFromUri)', error);
        }
        return filePath;
    };
    FileUtilProvider.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    FileUtilProvider.ctorParameters = function () { return [
        { type: Platform, },
        { type: File, },
    ]; };
    return FileUtilProvider;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de manipulação de arquivo
 */
var FileUtilModule = /** @class */ (function () {
    function FileUtilModule() {
    }
    FileUtilModule.decorators = [
        { type: NgModule, args: [{
                    providers: [
                        File,
                        FileUtilProvider
                    ]
                },] },
    ];
    return FileUtilModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

var __awaiter$2 = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator$2 = (undefined && undefined.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de geolocalização
 * com métodos que facilitam a solicitação de permissão e identificação de geolocalização
 */
var GeolocationUtilProvider = /** @class */ (function () {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * @param platform Serviço relacionado a plataforma
     * @param geolocation Serviço do plugin nativo de geolocalização
     * @param global Serviço global da aplicação
     */
    function GeolocationUtilProvider(platform, geolocation$$1, global) {
        var _this = this;
        this.geolocation = geolocation$$1;
        this.global = global;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then(function () { return _this.isCordova = platform.is('cordova'); });
    }
    /**
     * Método responsável por obter a posição corrente do usuário
     * @param {?} options
     * @return {?} Posição
     */
    GeolocationUtilProvider.prototype.getCurrentPosition = /**
     * Método responsável por obter a posição corrente do usuário
     * @param {?} options
     * @return {?} Posição
     */
    function (options) {
        return __awaiter$2(this, void 0, void 0, function () {
            var msg;
            return __generator$2(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.isCordova === false) {
                            msg = new BaseMessage({
                                literal: 'recursoNaoEstaDisponivelMsg'
                            });
                            this.global.msg.showMessage(msg);
                            throw new CriticalError('Platform is not cordova');
                        }
                        return [4 /*yield*/, this.geolocation.getCurrentPosition(options)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Método responsável por calcular a distância entre dois pontos através de suas latitudes e longitudes
     * @param {?} point1 Coordenadas do primeiro ponto
     * @param {?} point2 Coordenadas do segundo ponto
     * @return {?}
     */
    GeolocationUtilProvider.prototype.calculatePointsKMDistance = /**
     * Método responsável por calcular a distância entre dois pontos através de suas latitudes e longitudes
     * @param {?} point1 Coordenadas do primeiro ponto
     * @param {?} point2 Coordenadas do segundo ponto
     * @return {?}
     */
    function (point1, point2) {
        if (!this.validGeoPositionPoint(point1) || !this.validGeoPositionPoint(point2)) {
            throw new Error('ERROR - Invalid values');
        }
        if (point1.latitude === point2.latitude && point1.longitude === point2.longitude) {
            return 0;
        }
        var /** @type {?} */ radlat1 = this.degreesToRad(point1.latitude);
        var /** @type {?} */ radlat2 = this.degreesToRad(point2.latitude);
        var /** @type {?} */ theta = point1.longitude - point2.longitude;
        var /** @type {?} */ radtheta = this.degreesToRad(theta);
        var /** @type {?} */ dist = Math.sin(radlat1) * Math.sin(radlat2) + Math.cos(radlat1) * Math.cos(radlat2) * Math.cos(radtheta);
        if (dist > 1) {
            dist = 1;
        }
        dist = Math.acos(dist);
        dist = dist * 180 / Math.PI;
        dist = dist * 60 * 1.1515;
        // KM
        return dist * 1.609344;
    };
    /**
     * Converte os graus decimais em radianos
     * @param {?} degrees Graus decimais
     * @return {?}
     */
    GeolocationUtilProvider.prototype.degreesToRad = /**
     * Converte os graus decimais em radianos
     * @param {?} degrees Graus decimais
     * @return {?}
     */
    function (degrees) {
        return Math.PI * degrees / 180;
    };
    /**
     * Valida os valores do ponto recebido e retorna o resultado
     * @param {?} point Ponto contendo latitude e longitude
     * @return {?}
     */
    GeolocationUtilProvider.prototype.validGeoPositionPoint = /**
     * Valida os valores do ponto recebido e retorna o resultado
     * @param {?} point Ponto contendo latitude e longitude
     * @return {?}
     */
    function (point) {
        if (!point) {
            return false;
        }
        if (isNaN(point.latitude) || isNaN(point.longitude)) {
            return false;
        }
        if (point.latitude === null || point.longitude === null) {
            return false;
        }
        if (point.latitude > 90 || point.latitude < -90) {
            return false;
        }
        if (point.longitude > 180 || point.longitude < -180) {
            return false;
        }
        return true;
    };
    GeolocationUtilProvider.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    GeolocationUtilProvider.ctorParameters = function () { return [
        { type: Platform, },
        { type: Geolocation, },
        { type: GlobalService, },
    ]; };
    return GeolocationUtilProvider;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de geolocalização
 */
var GeolocationUtilModule = /** @class */ (function () {
    function GeolocationUtilModule() {
    }
    GeolocationUtilModule.decorators = [
        { type: NgModule, args: [{
                    providers: [
                        Geolocation,
                        GeolocationUtilProvider
                    ]
                },] },
    ];
    return GeolocationUtilModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

var __awaiter$3 = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator$3 = (undefined && undefined.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de requisição http
 * nativa do dispositivo com métodos que facilitam a utilização do plugin
 */
var HttpOauth2UtilProvider = /** @class */ (function () {
    /**
     * @ignore
     * Método construtor responsável por inicializar o serviço
     * @param platform Serviço relacionado a plataforma
     * @param file Serviço do plugin nativo de arquivo
     * @param events Serviço de eventos do Ionic
     */
    function HttpOauth2UtilProvider(platform, auth, http$$1) {
        var _this = this;
        this.auth = auth;
        this.http = http$$1;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then(function () { return (_this.isCordova = platform.is('cordova')); });
    }
    /**
     * Método responsável por realizar o upload de um arquivo
     * @template T Objeto de resposta do servidor quando sucesso
     * @param url URL do Endpoint de Upload
     * @param filePath URI do arquivo no dispositivo
     * @param name Nome do arquivo
     * @param body Informações no corpo da mensagem
     * @param headers Informações de cabeçalho da mensagem
     * @return {IHttpUtilResult} Resultado da requisição
     */
    /**
     * Método responsável por realizar o upload de um arquivo
     * @template T
     * @param {?} url URL do Endpoint de Upload
     * @param {?} filePath URI do arquivo no dispositivo
     * @param {?=} name Nome do arquivo
     * @param {?=} body Informações no corpo da mensagem
     * @param {?=} headers Informações de cabeçalho da mensagem
     * @return {?}
     */
    HttpOauth2UtilProvider.prototype.uploadFile = /**
     * Método responsável por realizar o upload de um arquivo
     * @template T
     * @param {?} url URL do Endpoint de Upload
     * @param {?} filePath URI do arquivo no dispositivo
     * @param {?=} name Nome do arquivo
     * @param {?=} body Informações no corpo da mensagem
     * @param {?=} headers Informações de cabeçalho da mensagem
     * @return {?}
     */
    function (url, filePath, name, body, headers) {
        return __awaiter$3(this, void 0, void 0, function () {
            var user, error_1, postHeaders, postBody, result, response, error_2;
            return __generator$3(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.isCordova === false) {
                            throw new CriticalError('Platform is not cordova (uploadFile)');
                        }
                        url = url.replace('{tenant}', this.auth.shortNameTenant || '');
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this.auth.getUser()];
                    case 2:
                        user = _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _a.sent();
                        return [3 /*break*/, 4];
                    case 4:
                        if (headers) {
                            postHeaders = Object.assign({}, {
                                Authorization: user ? user.token_type + " " + user.access_token : ''
                            }, headers);
                        }
                        else {
                            postHeaders = {
                                Authorization: user ? user.token_type + " " + user.access_token : ''
                            };
                        }
                        if (body) {
                            postBody = body;
                        }
                        else {
                            postBody = {};
                        }
                        if (name === undefined || name === '') {
                            name = 'file';
                        }
                        result = {
                            status: 0,
                            data: undefined,
                            error: undefined,
                            sourceResponse: undefined
                        };
                        _a.label = 5;
                    case 5:
                        _a.trys.push([5, 7, , 8]);
                        return [4 /*yield*/, this.http.uploadFile(url, postBody, postHeaders, filePath, name)];
                    case 6:
                        response = _a.sent();
                        result.sourceResponse = response;
                        if (response) {
                            if (response.hasOwnProperty('status')) {
                                result.status = response.status;
                            }
                            if (response.hasOwnProperty('data')) {
                                try {
                                    result.data = /** @type {?} */ (JSON.parse(response.data));
                                }
                                catch (/** @type {?} */ error) {
                                    result.error = error;
                                }
                            }
                        }
                        return [3 /*break*/, 8];
                    case 7:
                        error_2 = _a.sent();
                        result.error = error_2;
                        return [3 /*break*/, 8];
                    case 8: return [2 /*return*/, result];
                }
            });
        });
    };
    /**
     * Método que permite a alteração do modo de verificação do certificado SSL
     * @param mode Modo de verificação do certificado SSL
     */
    /**
     * Método que permite a alteração do modo de verificação do certificado SSL
     * @param {?} mode Modo de verificação do certificado SSL
     * @return {?}
     */
    HttpOauth2UtilProvider.prototype.setSSLCertMode = /**
     * Método que permite a alteração do modo de verificação do certificado SSL
     * @param {?} mode Modo de verificação do certificado SSL
     * @return {?}
     */
    function (mode) {
        return __awaiter$3(this, void 0, void 0, function () {
            return __generator$3(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.http.setSSLCertMode(mode)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    HttpOauth2UtilProvider.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    HttpOauth2UtilProvider.ctorParameters = function () { return [
        { type: Platform, },
        { type: AuthBaseService, },
        { type: HTTP, },
    ]; };
    return HttpOauth2UtilProvider;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de requisição http
 */
var HttpOauth2UtilModule = /** @class */ (function () {
    function HttpOauth2UtilModule() {
    }
    HttpOauth2UtilModule.decorators = [
        { type: NgModule, args: [{
                    providers: [
                        HTTP,
                        HttpOauth2UtilProvider
                    ]
                },] },
    ];
    return HttpOauth2UtilModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Classe que contém os eventos que o utilitário assina e publica
 */
var ImagePickerUtilEvents = /** @class */ (function () {
    function ImagePickerUtilEvents() {
    }
    /**
     * Evento publicado pelo utilitário quando uma ou mais imagens são
     * selecionadas e geram uma cópia no cache do aplicativo
     */
    ImagePickerUtilEvents.CACHE_FILE_CREATED = 'CacheFile:Created';
    return ImagePickerUtilEvents;
}());

var __awaiter$4 = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator$4 = (undefined && undefined.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de seleção
 * de imagens do dispositivo com métodos que facilitam a solicitação de permissão,
 * apresentação e seleção de imagens
 */
var ImagePickerUtilProvider = /** @class */ (function () {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * @param platform Serviço relacionado a plataforma
     * @param imagePicker Serviço do plugin nativo de galeria
     * @param global Serviço global da aplicação
     */
    function ImagePickerUtilProvider(platform, imagePicker$$1, global, events) {
        var _this = this;
        this.imagePicker = imagePicker$$1;
        this.global = global;
        this.events = events;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        /**
         * @ignore
         * Opções padrões para a galeria
         */
        this.defaultOptions = {
            outputType: OutputType.FILE_URL,
            quality: 60
        };
        platform.ready().then(function () { return _this.isCordova = platform.is('cordova'); });
    }
    /**
     * Método responsável por abrir o seletor de imagens e retornar o caminho dos arquivos salvos no cache
     * Este método publica o evento ImagePickerUtilEvents.CACHE_FILE_CREATED quando um ou mais arquivos são selecionados
     * e criados em cache (Mais informações sobre eventos {\@link https://ionicframework.com/docs/api/util/Events/|Ionic Events})
     * @param {?=} opt Customização das opções enviadas ao seletor de images
     * @return {?} Lista de imagens escolhidas
     */
    ImagePickerUtilProvider.prototype.chooseImage = /**
     * Método responsável por abrir o seletor de imagens e retornar o caminho dos arquivos salvos no cache
     * Este método publica o evento ImagePickerUtilEvents.CACHE_FILE_CREATED quando um ou mais arquivos são selecionados
     * e criados em cache (Mais informações sobre eventos {\@link https://ionicframework.com/docs/api/util/Events/|Ionic Events})
     * @param {?=} opt Customização das opções enviadas ao seletor de images
     * @return {?} Lista de imagens escolhidas
     */
    function (opt) {
        return __awaiter$4(this, void 0, void 0, function () {
            var msg, images, options, _i, _a, prop, hasPermission, msg, error_1, _b;
            return __generator$4(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        if (this.isCordova === false) {
                            msg = new BaseMessage({
                                literal: 'recursoNaoEstaDisponivelMsg'
                            });
                            this.global.msg.showMessage(msg);
                            throw new CriticalError('Platform is not cordova');
                        }
                        images = [];
                        _c.label = 1;
                    case 1:
                        _c.trys.push([1, 6, , 7]);
                        options = Object.assign({}, this.defaultOptions);
                        if (opt) {
                            for (_i = 0, _a = Object.keys(opt); _i < _a.length; _i++) {
                                prop = _a[_i];
                                options[prop] = opt[prop];
                            }
                        }
                        return [4 /*yield*/, this.requestReadPermission()];
                    case 2:
                        hasPermission = _c.sent();
                        if (!hasPermission) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.imagePicker.getPictures(options)];
                    case 3:
                        images = _c.sent();
                        (_b = this.events).publish.apply(_b, [ImagePickerUtilEvents.CACHE_FILE_CREATED].concat(images));
                        return [3 /*break*/, 5];
                    case 4:
                        msg = new BaseMessage({
                            literal: 'permissaoNaoConcedidaParaORecursoMsg'
                        });
                        this.global.msg.showMessage(msg);
                        throw new CriticalError('Permission denied');
                    case 5: return [3 /*break*/, 7];
                    case 6:
                        error_1 = _c.sent();
                        throw new CriticalError('Error in get pictures', error_1);
                    case 7: return [2 /*return*/, images];
                }
            });
        });
    };
    /**
     * Método responsável por requisitar permissão para acesso a galeria
     * @return {?} Indicação se possui ou não permissão
     */
    ImagePickerUtilProvider.prototype.requestReadPermission = /**
     * Método responsável por requisitar permissão para acesso a galeria
     * @return {?} Indicação se possui ou não permissão
     */
    function () {
        return __awaiter$4(this, void 0, void 0, function () {
            var hasPermission, permission, permissionAfterRequest, error_2;
            return __generator$4(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        hasPermission = false;
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 7, , 8]);
                        return [4 /*yield*/, this.imagePicker.hasReadPermission()];
                    case 2:
                        permission = _a.sent();
                        if (!!permission) return [3 /*break*/, 5];
                        return [4 /*yield*/, this.imagePicker.requestReadPermission()];
                    case 3:
                        _a.sent();
                        return [4 /*yield*/, this.imagePicker.hasReadPermission()];
                    case 4:
                        permissionAfterRequest = _a.sent();
                        hasPermission = permissionAfterRequest;
                        return [3 /*break*/, 6];
                    case 5:
                        hasPermission = true;
                        _a.label = 6;
                    case 6: return [3 /*break*/, 8];
                    case 7:
                        error_2 = _a.sent();
                        hasPermission = false;
                        return [3 /*break*/, 8];
                    case 8: return [2 /*return*/, hasPermission];
                }
            });
        });
    };
    ImagePickerUtilProvider.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    ImagePickerUtilProvider.ctorParameters = function () { return [
        { type: Platform, },
        { type: ImagePicker, },
        { type: GlobalService, },
        { type: Events, },
    ]; };
    return ImagePickerUtilProvider;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de galeria
 */
var ImagePickerUtilModule = /** @class */ (function () {
    function ImagePickerUtilModule() {
    }
    ImagePickerUtilModule.decorators = [
        { type: NgModule, args: [{
                    providers: [
                        ImagePicker,
                        ImagePickerUtilProvider
                    ]
                },] },
    ];
    return ImagePickerUtilModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de leitura de QR Code
 * com métodos que facilitam a solicitação de permissão e leitura do código
 */
var QrScannerUtilProvider = /** @class */ (function () {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * @param qrScanner Serviço do plugin nativo de leitura do QR-Code
     * @param global Serviço global da aplicação
     */
    function QrScannerUtilProvider(qrScanner$$1, global) {
        this.qrScanner = qrScanner$$1;
        this.global = global;
        /**
         * Propriedade que possui informações sobre o QR Code Scanner após ele estar
         * preparado, indicando assim se é possível ligar o flash ou trocar de camera,
         * por exemplo.
         */
        this.qrScannerLastStatus = new Subject();
    }
    /**
     * Método responsável por preparar o plugin para leitura do QR-Code
     * @return {?} Indicação da preparação
     */
    QrScannerUtilProvider.prototype.prepare = /**
     * Método responsável por preparar o plugin para leitura do QR-Code
     * @return {?} Indicação da preparação
     */
    function () {
        var _this = this;
        return new Promise(function (resolve) {
            _this.qrScanner.prepare()
                .then(function (status) {
                _this.qrScannerLastStatus.next(status);
                if (status.authorized) {
                    resolve(true);
                }
                else {
                    _this.global.msg.showMessage(new BaseMessage({
                        literal: 'permissaoNaoConcedidaParaORecursoMsg'
                    }));
                    resolve(false);
                }
            })
                .catch(function (error) {
                if (error && error.hasOwnProperty('name') && error.name === 'CAMERA_ACCESS_DENIED') {
                    _this.global.msg.showMessage(new BaseMessage({
                        literal: 'permissaoNaoConcedidaParaORecursoMsg'
                    }));
                }
                else {
                    _this.global.msg.showMessage(new BaseMessage({
                        literal: 'recursoNaoEstaDisponivelMsg'
                    }));
                }
                resolve(false);
            });
        });
    };
    /**
     * Método responsável por apresentar o leitor do QR-Code
     * @return {?} String com o conteúdo lido
     */
    QrScannerUtilProvider.prototype.show = /**
     * Método responsável por apresentar o leitor do QR-Code
     * @return {?} String com o conteúdo lido
     */
    function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (!_this.qrScannerLastStatus) {
                reject(undefined);
            }
            _this.scanSub = _this.qrScanner.scan().subscribe(function (result) {
                _this.qrScanner.hide();
                _this.scanSub.unsubscribe(); // stop scanning
                resolve(result);
            }, function (error) {
                reject(error);
            });
            _this.qrScanner.show()
                .then(function () { })
                .catch(function (error) { return reject(error); });
        });
    };
    /**
     * Método responsável por esconder a camera da leitura do QR-Code
     * @return {?}
     */
    QrScannerUtilProvider.prototype.hide = /**
     * Método responsável por esconder a camera da leitura do QR-Code
     * @return {?}
     */
    function () {
        var _this = this;
        this.qrScanner.getStatus().then(function (status) {
            if (status && status.showing) {
                _this.qrScanner.hide().then(function (statusAfter) {
                    _this.qrScannerLastStatus.next(statusAfter);
                });
            }
            if (status && status.scanning) {
                _this.scanSub.unsubscribe();
            }
        });
    };
    /**
     * Método responsável por ligar ou desligar o flash
     * @return {?}
     */
    QrScannerUtilProvider.prototype.flash = /**
     * Método responsável por ligar ou desligar o flash
     * @return {?}
     */
    function () {
        var _this = this;
        this.qrScanner.getStatus().then(function (status) {
            if (status && status.scanning && status.canEnableLight) {
                if (status.lightEnabled) {
                    _this.qrScanner.disableLight()
                        .then(function (statusAfter) {
                        _this.qrScannerLastStatus.next(statusAfter);
                    })
                        .catch(function () { });
                }
                else {
                    _this.qrScanner.enableLight()
                        .then(function (statusAfter) {
                        _this.qrScannerLastStatus.next(statusAfter);
                    })
                        .catch(function () { });
                }
            }
        }).catch(function () { });
    };
    /**
     * Método responsável por alternar entre a camera frontal e a camera traseira
     * @return {?}
     */
    QrScannerUtilProvider.prototype.changeCamera = /**
     * Método responsável por alternar entre a camera frontal e a camera traseira
     * @return {?}
     */
    function () {
        var _this = this;
        this.qrScanner.getStatus().then(function (status) {
            if (status && status.scanning && status.canChangeCamera) {
                if (status.currentCamera === 0) {
                    // Back camera
                    _this.qrScanner.useFrontCamera()
                        .then(function (statusAfter) {
                        _this.qrScannerLastStatus.next(statusAfter);
                    })
                        .catch(function () { });
                }
                else {
                    _this.qrScanner.useBackCamera()
                        .then(function (statusAfter) {
                        _this.qrScannerLastStatus.next(statusAfter);
                    })
                        .catch(function () { });
                }
            }
        }).catch(function () { });
    };
    /**
     * Método responsável por destruir o plugin nativo desabilitando a leitura do QR-Code até
     * que o plugin seja novamente preparado
     * @return {?}
     */
    QrScannerUtilProvider.prototype.destroy = /**
     * Método responsável por destruir o plugin nativo desabilitando a leitura do QR-Code até
     * que o plugin seja novamente preparado
     * @return {?}
     */
    function () {
        var _this = this;
        this.qrScanner.getStatus().then(function (status) {
            if (status) {
                if (status.lightEnabled) {
                    _this.qrScanner.disableLight()
                        .then(function () { })
                        .catch(function () { });
                }
                _this.qrScanner.destroy()
                    .then(function () { })
                    .catch(function () { });
            }
        });
    };
    QrScannerUtilProvider.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    QrScannerUtilProvider.ctorParameters = function () { return [
        { type: QRScanner, },
        { type: GlobalService, },
    ]; };
    return QrScannerUtilProvider;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de leitura de QR-Code
 */
var QRScannerUtilModule = /** @class */ (function () {
    function QRScannerUtilModule() {
    }
    QRScannerUtilModule.decorators = [
        { type: NgModule, args: [{
                    providers: [
                        QRScanner,
                        QrScannerUtilProvider
                    ]
                },] },
    ];
    return QRScannerUtilModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de reconhecimento de voz
 * com métodos que facilitam a solicitação de permissão e reconhecimento
 */
var SpeechRecognitionUtilProvider = /** @class */ (function () {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * @param platform Serviço relacionado a plataforma
     * @param speechRecognition Serviço do plugin nativo de reconhecimento de voz
     * @param global Serviço global da aplicação
     */
    function SpeechRecognitionUtilProvider(platform, speechRecognition$$1, global) {
        var _this = this;
        this.speechRecognition = speechRecognition$$1;
        this.global = global;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then(function () { return _this.isCordova = platform.is('cordova'); });
    }
    /**
     * Método responsável por habilitar o reconheicimento de voz
     * @return {?} Lista com os textos reconhecidos (geralmente o primeiro item é o mais provável)
     */
    SpeechRecognitionUtilProvider.prototype.listening = /**
     * Método responsável por habilitar o reconheicimento de voz
     * @return {?} Lista com os textos reconhecidos (geralmente o primeiro item é o mais provável)
     */
    function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.verifyPermission().then(function (hasPermission) {
                if (hasPermission === false) {
                    _this.showToast('permissaoNaoConcedidaParaORecursoMsg');
                    resolve([]);
                }
                else {
                    _this.speechRecognition.isRecognitionAvailable()
                        .then(function (available) {
                        if (available === false) {
                            _this.showToast('recursoNaoEstaDisponivelMsg');
                            resolve([]);
                            return;
                        }
                        _this.speechRecognition.startListening(_this.options)
                            .subscribe(function (matches) { return resolve(matches); }, function (onerror) {
                            if (onerror && onerror instanceof Error && onerror.message !== '0') {
                                _this.showToast('erroAoUtilizarRecursoMsg');
                            }
                            reject(onerror);
                        });
                    }).catch(function (error) {
                        _this.showToast('recursoNaoEstaDisponivelMsg');
                        reject(error);
                    });
                }
            }).catch(function (error) {
                reject(error);
            });
        });
    };
    /**
     * @ignore
     * Método responsável por verificar se há permissão e se não houver, solicitar
     * @return {?} Indicação se há ou não permissão
     */
    SpeechRecognitionUtilProvider.prototype.verifyPermission = /**
     * @ignore
     * Método responsável por verificar se há permissão e se não houver, solicitar
     * @return {?} Indicação se há ou não permissão
     */
    function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (_this.isCordova === false) {
                _this.showToast('recursoNaoEstaDisponivelMsg');
                reject('Cordova is not available.');
                return;
            }
            _this.speechRecognition.hasPermission().then(function (hasPermission) {
                if (hasPermission) {
                    resolve(hasPermission);
                }
                else {
                    _this.requestPermission()
                        .then(function (requestPermissionSuccessful) { return resolve(requestPermissionSuccessful); });
                }
            }).catch(function () { return resolve(false); });
        });
    };
    /**
     * @ignore
     * Método responsável por solicitar permissão de voz
     * @return {?} Indicação se a permissão foi ou não concedida
     */
    SpeechRecognitionUtilProvider.prototype.requestPermission = /**
     * @ignore
     * Método responsável por solicitar permissão de voz
     * @return {?} Indicação se a permissão foi ou não concedida
     */
    function () {
        var _this = this;
        return new Promise(function (resolve) {
            _this.speechRecognition.requestPermission()
                .then(function () { return resolve(true); }, function () { return resolve(false); }).catch(function () { return resolve(false); });
        });
    };
    /**
     * @ignore
     * Método para apresentar mensagens
     * @param {?} literalMessage Literal da mensagem
     * @return {?}
     */
    SpeechRecognitionUtilProvider.prototype.showToast = /**
     * @ignore
     * Método para apresentar mensagens
     * @param {?} literalMessage Literal da mensagem
     * @return {?}
     */
    function (literalMessage) {
        this.global.msg.showMessage(new BaseMessage({
            literal: literalMessage
        }));
    };
    SpeechRecognitionUtilProvider.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    SpeechRecognitionUtilProvider.ctorParameters = function () { return [
        { type: Platform, },
        { type: SpeechRecognition, },
        { type: GlobalService, },
    ]; };
    return SpeechRecognitionUtilProvider;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de reconhecimento de voz
 */
var SpeechRecognitionUtilModule = /** @class */ (function () {
    function SpeechRecognitionUtilModule() {
    }
    SpeechRecognitionUtilModule.decorators = [
        { type: NgModule, args: [{
                    providers: [
                        SpeechRecognition,
                        SpeechRecognitionUtilProvider
                    ]
                },] },
    ];
    return SpeechRecognitionUtilModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

var __awaiter$5 = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator$5 = (undefined && undefined.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Classe responsável pelo gerenciamento de variável em memória/storage do dispositivo
 * Ao utilizar os métodos da classe para atualizar o valor em memória, por padrão,
 * o valor é refletido no storage
 * @template T
 */
var StorageUtil = /** @class */ (function () {
    /**
     * Construtor da classe
     * @param storage Recebe o Storage via Angular Dependency Injection
     */
    function StorageUtil(storage$$1) {
        this.storage = storage$$1;
        /**
         * Campo id para facilitar buscas
         */
        this._id = '';
        /**
         * Conteúdo a ser refletido
         */
        this._data = {};
        /**
         * Indica que a classe já foi inicializada corretamente
         */
        this._init = false;
        /**
         * Se uma operação que trava o registro está em andamento
         */
        this._lock = false;
    }
    Object.defineProperty(StorageUtil.prototype, "id", {
        get: /**
         * @return {?}
         */
        function () {
            return this._id;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StorageUtil.prototype, "exclusive", {
        get: /**
         * @return {?}
         */
        function () {
            return this._exclusive;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StorageUtil.prototype, "suffixFn", {
        get: /**
         * @return {?}
         */
        function () {
            if (!this._suffixFn) {
                return function () { return ''; };
            }
            else {
                return this._suffixFn;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StorageUtil.prototype, "initFn", {
        get: /**
         * @return {?}
         */
        function () {
            return this._initFn;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StorageUtil.prototype, "name", {
        get: /**
         * @return {?}
         */
        function () {
            return this._name;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StorageUtil.prototype, "data", {
        get: /**
         * @return {?}
         */
        function () {
            if (!this.isInit) {
                throw new Error("Storage: " + this.name + " est\u00E1 sendo lido sem ser inicializado.");
            }
            return this.getCurrentSuffixData();
        },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            if (!this.isInit) {
                throw new Error("Storage: " + this.name + " est\u00E1 sendo definido sem ser inicializado.");
            }
            var /** @type {?} */ suffix = this.currentSuffix;
            this._data[suffix] = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StorageUtil.prototype, "currentSuffix", {
        /**
         * Sufixo corrente quando storage exclusivo
         */
        get: /**
         * Sufixo corrente quando storage exclusivo
         * @return {?}
         */
        function () {
            var /** @type {?} */ suffix = this.suffixFn();
            if (!suffix || suffix === '') {
                suffix = StorageUtil.SUFFIX_DEFAULT;
            }
            return suffix;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StorageUtil.prototype, "isInit", {
        get: /**
         * @return {?}
         */
        function () {
            return this._init;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(StorageUtil.prototype, "isLock", {
        get: /**
         * @return {?}
         */
        function () {
            return this._lock;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Configura e realiza a leitura de dados do storage
     * @param { name:string, id? :string, exclusive?: boolean, suffixFn?: Function, lazy?: boolean, initFn?: Function}
     */
    /**
     * Configura e realiza a leitura de dados do storage
     * @param {?=} __0
     * @return {?}
     */
    StorageUtil.prototype.config = /**
     * Configura e realiza a leitura de dados do storage
     * @param {?=} __0
     * @return {?}
     */
    function (_a) {
        var _b = _a === void 0 ? {
            name: '',
            id: '',
            exclusive: false,
            suffixFn: function () { return ''; },
            lazy: false,
            initFn: initFn
        } : _a, name = _b.name, id = _b.id, exclusive = _b.exclusive, suffixFn = _b.suffixFn, lazy = _b.lazy, initFn = _b.initFn;
        if (!name || name === '') {
            throw new Error('Storage necessita de um nome (name)');
        }
        if (exclusive && !suffixFn) {
            throw new Error('Storage exclusivo necessita de uma função para retornar o sufixo (suffixFn)');
        }
        this._name = name;
        this._id = id;
        this._exclusive = exclusive;
        this._suffixFn = suffixFn;
        this._initFn = initFn;
        if (lazy) {
            return Promise.resolve(undefined);
        }
        else {
            return this.read();
        }
    };
    /**
     * Atualiza o storage para refletir o valor contido em memória
     */
    /**
     * Atualiza o storage para refletir o valor contido em memória
     * @return {?}
     */
    StorageUtil.prototype.updateStorage = /**
     * Atualiza o storage para refletir o valor contido em memória
     * @return {?}
     */
    function () {
        return __awaiter$5(this, void 0, void 0, function () {
            var _this = this;
            return __generator$5(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.isInit) {
                            throw new Error('Necessário chamar algum dos métodos de leitura (get,read)');
                        }
                        return [4 /*yield*/, UtilsFn.lock(function () { return _this._lock; })];
                    case 1:
                        _a.sent();
                        this._lock = true;
                        return [4 /*yield*/, this.storage.set(this.name, this._data)
                                .then(function () {
                                _this._lock = false;
                                return _this.data;
                            })
                                .catch(function (err) {
                                _this._lock = false;
                                console.error(err);
                                return undefined;
                            })];
                    case 2: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Ferramenta utilizada para encontrar algum valor em um subnível
     * @param search
     * @param fields
     * @example
     * storage.data = [{ nome: 'joao', endereco: {rua: 'qualquer', numero: 123}}]
     * storage.find(123,'endereco','numero').then(endereco =>{
     * endereco.numero = 234
     * storage.updateStorage()})
     */
    /**
     * Ferramenta utilizada para encontrar algum valor em um subnível
     * \@example
     * storage.data = [{ nome: 'joao', endereco: {rua: 'qualquer', numero: 123}}]
     * storage.find(123,'endereco','numero').then(endereco =>{
     * endereco.numero = 234
     * storage.updateStorage()})
     * @param {?} search
     * @param {...?} fields
     * @return {?}
     */
    StorageUtil.prototype.find = /**
     * Ferramenta utilizada para encontrar algum valor em um subnível
     * \@example
     * storage.data = [{ nome: 'joao', endereco: {rua: 'qualquer', numero: 123}}]
     * storage.find(123,'endereco','numero').then(endereco =>{
     * endereco.numero = 234
     * storage.updateStorage()})
     * @param {?} search
     * @param {...?} fields
     * @return {?}
     */
    function (search) {
        var fields = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            fields[_i - 1] = arguments[_i];
        }
        return __awaiter$5(this, void 0, void 0, function () {
            var _this = this;
            var array;
            return __generator$5(this, function (_a) {
                array = this.data;
                array.find(function (element) {
                    var /** @type {?} */ ret = _this.recursiveField(fields, element) === search;
                    if (ret) {
                        fields.pop();
                        if (fields.length > 0) {
                            return _this.recursiveField(fields, element);
                        }
                        else {
                            return element;
                        }
                    }
                    return ret;
                });
                throw new Error('Item não encontrado no storage: ' + search);
            });
        });
    };
    /**
     * Atualiza a variavel em memória e reflete no storage.
     * Pode-se acessar um subnível passando no fields o nível que deseja acessar,
     * @example
     * // exemplo 1:
     * storage.data == {campoa:'valor', campob:{ campox: 'valor2'}}
     * storage.set('novovalor',true,'campob','campox')
     * // Resultado objeto: {campoa:'valor', campob:{ campox: 'novovalor'}}
     * // exemplo 2:
     * storage.data == [ {campoa:'valor', campob:{ campox: 'valor2'}}, {campoa:'valor', campob:{ campox: 'valork'}} ]
     * storage.set('novovalorx',true,1,'campob','campox')
     * // Caso atualizar o storage.data diretamente, chamar updateStorage para refletir os dados no storage
     * @param newData
     * @param save
     * @param fields
     */
    /**
     * Atualiza a variavel em memória e reflete no storage.
     * Pode-se acessar um subnível passando no fields o nível que deseja acessar,
     * \@example
     * // exemplo 1:
     * storage.data == {campoa:'valor', campob:{ campox: 'valor2'}}
     * storage.set('novovalor',true,'campob','campox')
     * // Resultado objeto: {campoa:'valor', campob:{ campox: 'novovalor'}}
     * // exemplo 2:
     * storage.data == [ {campoa:'valor', campob:{ campox: 'valor2'}}, {campoa:'valor', campob:{ campox: 'valork'}} ]
     * storage.set('novovalorx',true,1,'campob','campox')
     * // Caso atualizar o storage.data diretamente, chamar updateStorage para refletir os dados no storage
     * @param {?} newData
     * @param {?=} save
     * @param {...?} fields
     * @return {?}
     */
    StorageUtil.prototype.set = /**
     * Atualiza a variavel em memória e reflete no storage.
     * Pode-se acessar um subnível passando no fields o nível que deseja acessar,
     * \@example
     * // exemplo 1:
     * storage.data == {campoa:'valor', campob:{ campox: 'valor2'}}
     * storage.set('novovalor',true,'campob','campox')
     * // Resultado objeto: {campoa:'valor', campob:{ campox: 'novovalor'}}
     * // exemplo 2:
     * storage.data == [ {campoa:'valor', campob:{ campox: 'valor2'}}, {campoa:'valor', campob:{ campox: 'valork'}} ]
     * storage.set('novovalorx',true,1,'campob','campox')
     * // Caso atualizar o storage.data diretamente, chamar updateStorage para refletir os dados no storage
     * @param {?} newData
     * @param {?=} save
     * @param {...?} fields
     * @return {?}
     */
    function (newData, save) {
        if (save === void 0) { save = true; }
        var fields = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            fields[_i - 2] = arguments[_i];
        }
        return __awaiter$5(this, void 0, void 0, function () {
            var field, data;
            return __generator$5(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.isInit) {
                            throw new Error('Necessário chamar algum dos métodos de leitura (get,read)');
                        }
                        if (fields.length === 0) {
                            this.data = newData;
                        }
                        else {
                            field = fields.pop();
                            data = this.recursiveField(fields, this.data);
                            data[field] = newData;
                        }
                        if (!save) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.updateStorage()];
                    case 1: return [2 /*return*/, _a.sent()];
                    case 2: return [2 /*return*/, this.data];
                }
            });
        });
    };
    /**
     * Faz a leitura do storage se ainda não foi feita ou da variável de memória
     * @param search Busca em campo que representa uma chave primária do objeto
     */
    /**
     * Faz a leitura do storage se ainda não foi feita ou da variável de memória
     * @param {?} search Busca em campo que representa uma chave primária do objeto
     * @return {?}
     */
    StorageUtil.prototype.get = /**
     * Faz a leitura do storage se ainda não foi feita ou da variável de memória
     * @param {?} search Busca em campo que representa uma chave primária do objeto
     * @return {?}
     */
    function (search) {
        return __awaiter$5(this, void 0, void 0, function () {
            var _this = this;
            var array;
            return __generator$5(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.read()];
                    case 1:
                        _a.sent();
                        if (this.id && search) {
                            try {
                                array = this.data;
                                return [2 /*return*/, array.find(function (element) {
                                        return element[_this.id] === search;
                                    })];
                            }
                            catch (/** @type {?} */ error) {
                                throw error;
                            }
                        }
                        else {
                            return [2 /*return*/, this.data];
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Faz a leitura dos dados que estão no storage
     */
    /**
     * Faz a leitura dos dados que estão no storage
     * @return {?}
     */
    StorageUtil.prototype.read = /**
     * Faz a leitura dos dados que estão no storage
     * @return {?}
     */
    function () {
        return __awaiter$5(this, void 0, void 0, function () {
            var _this = this;
            var suffix;
            return __generator$5(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.name) {
                            throw new Error('Necessário chamar o método config');
                        }
                        return [4 /*yield*/, UtilsFn.lock(function () { return _this._lock; })];
                    case 1:
                        _a.sent();
                        suffix = this.currentSuffix;
                        if (!this._data.hasOwnProperty(suffix)) return [3 /*break*/, 2];
                        return [2 /*return*/, this.data];
                    case 2:
                        if (!this.isInit) return [3 /*break*/, 3];
                        this._data[suffix] = this.init(null);
                        return [3 /*break*/, 5];
                    case 3: return [4 /*yield*/, this.storage.get(this.name)
                            .then(function (value) {
                            return _this.updateData(value);
                        }).catch(function (err) {
                            console.error(err);
                            return undefined;
                        })];
                    case 4: return [2 /*return*/, _a.sent()];
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Reinicializa o storage, persistindo os dados de inicialização
     */
    /**
     * Reinicializa o storage, persistindo os dados de inicialização
     * @return {?}
     */
    StorageUtil.prototype.reset = /**
     * Reinicializa o storage, persistindo os dados de inicialização
     * @return {?}
     */
    function () {
        return __awaiter$5(this, void 0, void 0, function () {
            return __generator$5(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.updateData({});
                        return [4 /*yield*/, this.read()];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.updateStorage()];
                    case 2:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Remove um item de um array, atualiza no storage e retorna uma promessa de item atualizado.
     * @param search
     */
    /**
     * Remove um item de um array, atualiza no storage e retorna uma promessa de item atualizado.
     * @param {?} search
     * @return {?}
     */
    StorageUtil.prototype.delete = /**
     * Remove um item de um array, atualiza no storage e retorna uma promessa de item atualizado.
     * @param {?} search
     * @return {?}
     */
    function (search) {
        var _this = this;
        if (!this.isInit) {
            throw new Error('Necessário chamar algum dos métodos de leitura (get,read)');
        }
        if (this.id && search) {
            return new Promise(function (resolve, reject) {
                var /** @type {?} */ array = _this.data;
                var /** @type {?} */ index = array.findIndex(function (element) {
                    return element[_this.id] === search;
                });
                if (index > -1) {
                    array.splice(index, 1);
                    _this.set(array).then(function (v) {
                        resolve(_this.data);
                    });
                }
                else {
                    reject("Registro " + search + " n\u00E3o encontrado.");
                }
            });
        }
    };
    /**
     * Atualiza o valor em memória lida do storage
     * @param {?} value
     * @return {?}
     */
    StorageUtil.prototype.updateData = /**
     * Atualiza o valor em memória lida do storage
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (value === undefined || value === null || typeof value !== 'object') {
            this._data = {};
        }
        else {
            this._data = value;
        }
        this._init = true;
        return this.data;
    };
    /**
     * Inicializa o armazenamento com base na função que o usuário informou
     * @param {?} data
     * @return {?}
     */
    StorageUtil.prototype.init = /**
     * Inicializa o armazenamento com base na função que o usuário informou
     * @param {?} data
     * @return {?}
     */
    function (data) {
        if ((data === undefined || data === null) && this.initFn) {
            data = this.initFn();
        }
        return data;
    };
    /**
     * Função recursíva para encontrar subníveis automáticamente
     * @param {?} fields
     * @param {?} data a informação
     * @param {?=} level nível de iteração
     * @return {?}
     */
    StorageUtil.prototype.recursiveField = /**
     * Função recursíva para encontrar subníveis automáticamente
     * @param {?} fields
     * @param {?} data a informação
     * @param {?=} level nível de iteração
     * @return {?}
     */
    function (fields, data, level) {
        if (level === void 0) { level = 0; }
        if (level < fields.length - 1) {
            return this.recursiveField(fields, data[fields[level]], ++level);
        }
        return data[fields[level]];
    };
    /**
     * Recupera os dados do sufíxo atual
     * @return {?}
     */
    StorageUtil.prototype.getCurrentSuffixData = /**
     * Recupera os dados do sufíxo atual
     * @return {?}
     */
    function () {
        var /** @type {?} */ suffix = this.currentSuffix;
        if (this._data.hasOwnProperty(suffix) === false) {
            this._data[suffix] = this.init(null);
        }
        return this._data[suffix];
    };
    /**
     * Sufixo padrão
     */
    StorageUtil.SUFFIX_DEFAULT = '__stDefault0__';
    return StorageUtil;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

var __awaiter$6 = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator$6 = (undefined && undefined.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Gerenciador de conexão de banco de dados sqlite
 * @abstract
 */
var SqLiteUtilBaseProvider = /** @class */ (function () {
    function SqLiteUtilBaseProvider(sqlLite, platform) {
        this.sqlLite = sqlLite;
        this.platform = platform;
    }
    /**
     * Método para auxiliar a criar uma conexão com o banco sqlite
     * @param name Nome do banco
     * @param location Localização do banco
     * @memberof DbConnectionBaseProvider
     */
    /**
     * Método para auxiliar a criar uma conexão com o banco sqlite
     * \@memberof DbConnectionBaseProvider
     * @param {?} name Nome do banco
     * @param {?} location Localização do banco
     * @return {?}
     */
    SqLiteUtilBaseProvider.prototype.createConn = /**
     * Método para auxiliar a criar uma conexão com o banco sqlite
     * \@memberof DbConnectionBaseProvider
     * @param {?} name Nome do banco
     * @param {?} location Localização do banco
     * @return {?}
     */
    function (name, location) {
        return __awaiter$6(this, void 0, void 0, function () {
            return __generator$6(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.platform.ready()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, this.sqlLite.create({ name: name, location: location })];
                }
            });
        });
    };
    return SqLiteUtilBaseProvider;
}());

var __extends$7 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __awaiter$7 = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator$7 = (undefined && undefined.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Gerenciador de conexão de banco de dados sqlite
 */
var SqLiteUtilProvider = /** @class */ (function (_super) {
    __extends$7(SqLiteUtilProvider, _super);
    function SqLiteUtilProvider(sqlLite, platform) {
        var _this = _super.call(this, sqlLite, platform) || this;
        _this.connections = new Array();
        _this.creating = new Array();
        return _this;
    }
    /**
     * Recupera uma conexão de banco de dados
     *
     * @param {?} name o banco de dados desejado
     * @param {?=} location
     * @return {?} Um objeto de conexão com o banco
     */
    SqLiteUtilProvider.prototype.getSqLiteConn = /**
     * Recupera uma conexão de banco de dados
     *
     * @param {?} name o banco de dados desejado
     * @param {?=} location
     * @return {?} Um objeto de conexão com o banco
     */
    function (name, location) {
        if (location === void 0) { location = 'default'; }
        return __awaiter$7(this, void 0, void 0, function () {
            var _this = this;
            var element, sqlConn, idx;
            return __generator$7(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, UtilsFn.lock(function () { return _this.creating.find(function (x) { return x.name === name && x.location === location; }); })];
                    case 1:
                        _a.sent();
                        element = this.connections.find(function (x) { return x.name === name && x.location === location; });
                        if (!!element) return [3 /*break*/, 3];
                        this.creating.push({ name: name, location: location, conn: undefined });
                        return [4 /*yield*/, this.createConn(name, location)];
                    case 2:
                        sqlConn = _a.sent();
                        this.connections.push({ name: name, location: location, conn: sqlConn });
                        idx = this.creating.findIndex(function (x) { return x.name === name && x.location === location; });
                        if (idx !== -1) {
                            this.creating.splice(idx, 1);
                        }
                        return [3 /*break*/, 4];
                    case 3:
                        sqlConn = element.conn;
                        _a.label = 4;
                    case 4: return [2 /*return*/, sqlConn];
                }
            });
        });
    };
    SqLiteUtilProvider.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    SqLiteUtilProvider.ctorParameters = function () { return [
        { type: SQLite, },
        { type: Platform, },
    ]; };
    return SqLiteUtilProvider;
}(SqLiteUtilBaseProvider));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Módulo que controla as conexões com banco sqlite
 * e provê conexão com o serviço SqLite
 * Usar SqLiteUtilModule.forRoot() ao importar
 * @export
 */
var SqLiteUtilModule = /** @class */ (function () {
    function SqLiteUtilModule() {
    }
    /**
     * @return {?}
     */
    SqLiteUtilModule.forRoot = /**
     * @return {?}
     */
    function () {
        return {
            ngModule: SqLiteUtilModule,
            providers: [
                SQLite,
                {
                    provide: SqLiteUtilBaseProvider,
                    useClass: SqLiteUtilProvider
                }
            ]
        };
    };
    SqLiteUtilModule.decorators = [
        { type: NgModule, args: [{
                    providers: []
                },] },
    ];
    return SqLiteUtilModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Classe abstrata que todo recurso de sincronização deve implementar,
 * independente do sentido
 * Para registrar o serviço em que a sincronização automática reconheça:
 * export const entidadeToken = new InjectionToken<entidadeProvider>('');
 * export class entidadeProvider extends SyncHandler{}
 * {
 * provide: entidadeToken,
 * useClass: entidadeProvider
 * }
 * @abstract
 */
var SyncHandler = /** @class */ (function () {
    function SyncHandler() {
    }
    return SyncHandler;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Classe que contém todos os eventos disponíbilizados pelo serviço gerenciador de
 * sincronismo
 */
var SyncEvents = /** @class */ (function () {
    function SyncEvents() {
    }
    /**
     * Evento que o sincronizador dispara ao terminar um sincronismo.
     * Deve-se filtrar pelo nome da entidade
     */
    SyncEvents.GOT_ENTITY_SYNC_DATA = 'Sync:GotEntitySyncData';
    return SyncEvents;
}());

var __awaiter$8 = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator$8 = (undefined && undefined.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var SYNC_CONFIG = new InjectionToken('SYNC_CONFIG: ISyncConfig token de configuração da sincronização');
/**
 * Serviço provedor de sincronismo utilizando serviços externos
 */
var SyncProvider = /** @class */ (function () {
    /**
     * Construtor da classe
     * @param platform Platform do ionic
     * @param authService Serviço de autenticação
     * @param storage Serviço de armazenamento do ionic
     * @param config Token de configuração do serviço
     * @param network Serviço de rede do ionic
     * @param events Serviço de eventos do ionic
     * @param injector Injetor de dependências do angular
     */
    function SyncProvider(platform, storage$$1, config, network$$1, events, injector) {
        var _this = this;
        this.storage = storage$$1;
        this.config = config;
        this.network = network$$1;
        this.events = events;
        this.injector = injector;
        /**
         * @ignore
         */
        this._syncing = false;
        platform.ready().then(function () { return __awaiter$8(_this, void 0, void 0, function () {
            var _this = this;
            return __generator$8(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!platform.is('cordova')) return [3 /*break*/, 2];
                        this.syncStorage = new StorageUtil(this.storage);
                        return [4 /*yield*/, this.syncStorage.config({
                                name: 'SyncEntitiesState',
                                exclusive: true,
                                lazy: true,
                                suffixFn: function () { return _this.storageName; },
                                initFn: function () {
                                    var /** @type {?} */ init = [];
                                    config.entities.forEach(function (e) { return init.push(_this.newState(e)); });
                                    return init;
                                }
                            })];
                    case 1:
                        _a.sent();
                        this.setDefaults();
                        this.subscribeEvents();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        }); });
    }
    Object.defineProperty(SyncProvider.prototype, "isSyncing", {
        get: /**
         * Se a sincronização está acontecendo
         * @return {?}
         */
        function () {
            return this._syncing;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SyncProvider.prototype, "storageNameFunction", {
        set: /**
         * @param {?} storageNameFunction
         * @return {?}
         */
        function (storageNameFunction) {
            this._storageNameFunction = storageNameFunction;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SyncProvider.prototype, "storageName", {
        /**
         * Atributo que retorna o nome do storage
         */
        get: /**
         * Atributo que retorna o nome do storage
         * @return {?}
         */
        function () {
            if (this._storageNameFunction) {
                return this._storageNameFunction();
            }
            else {
                return function () { return 'sync'; };
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SyncProvider.prototype, "autoWatchEnabled", {
        get: /**
         * Retorna se o sincronismo está ativo
         * @return {?}
         */
        function () {
            var _this = this;
            if (this._currentNetworkType === 'none') {
                return false;
            }
            if (!this.config.autoWatch) {
                return false;
            }
            if (this.config.autoWatch.enabledOnNetwork === 'any' || this.config.autoWatch.enabledOnNetwork === this._currentNetworkType) {
                return true;
            }
            else {
                var /** @type {?} */ array = /** @type {?} */ (this.config.autoWatch.enabledOnNetwork);
                if (array.findIndex(function (x) { return x === 'any' || x === _this._currentNetworkType; }) !== -1) {
                    return true;
                }
            }
            return false;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Dispara o sincronismo de todas as entidades registradas.
     * Consulte isSyncing para saber se a sincronização está em andamento.
     * @return {?}
     */
    SyncProvider.prototype.syncAll = /**
     * Dispara o sincronismo de todas as entidades registradas.
     * Consulte isSyncing para saber se a sincronização está em andamento.
     * @return {?}
     */
    function () {
        return __awaiter$8(this, void 0, void 0, function () {
            var error_1;
            return __generator$8(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this._syncing === false)) return [3 /*break*/, 4];
                        this._syncing = true;
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this.syncStorage.read()];
                    case 2:
                        _a.sent();
                        this.syncAllByOrder(this.config.entities);
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _a.sent();
                        console.error('Sync autowatch stopped. Reason', error_1);
                        this._syncing = false;
                        this.stopAutoWatch();
                        return [3 /*break*/, 4];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Função recursiva que dispara a sincronização de entidades conforme ordem recebida
     * @param {?} entities Lista com todas as entidades
     * @param {?=} order Ordem da execução
     * @return {?}
     */
    SyncProvider.prototype.syncAllByOrder = /**
     * Função recursiva que dispara a sincronização de entidades conforme ordem recebida
     * @param {?} entities Lista com todas as entidades
     * @param {?=} order Ordem da execução
     * @return {?}
     */
    function (entities, order) {
        if (order === void 0) { order = 0; }
        return __awaiter$8(this, void 0, void 0, function () {
            var entitiesByOrder, result;
            return __generator$8(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        entitiesByOrder = [];
                        entities.forEach(function (e) {
                            if ((order === 0 && (e.order === undefined || e.order === 0)) ||
                                (order !== 0 && (e.order === order))) {
                                entitiesByOrder.push(e);
                            }
                        });
                        if (!(entitiesByOrder.length > 0)) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.syncAllEntities(entitiesByOrder)];
                    case 1:
                        result = _a.sent();
                        // Aborta demais sincronizações
                        if (!result) {
                            return [2 /*return*/];
                        }
                        _a.label = 2;
                    case 2:
                        if (entities.some(function (e) { return e.order > order; })) {
                            this.syncAllByOrder(entities, order + 1);
                        }
                        else {
                            this._syncing = false;
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Dispara o sincronismo de todas as entidades registradas conforme ordem
     * Publica o evento SyncEvents.GOT_ENTITY_SYNC_DATA da finalização da sincronização da entidade.
     * @param {?} entities Lista com as entidades que deverão ser sincronizadas
     * @return {?} Indicação se ocorreu com sucesso
     */
    SyncProvider.prototype.syncAllEntities = /**
     * Dispara o sincronismo de todas as entidades registradas conforme ordem
     * Publica o evento SyncEvents.GOT_ENTITY_SYNC_DATA da finalização da sincronização da entidade.
     * @param {?} entities Lista com as entidades que deverão ser sincronizadas
     * @return {?} Indicação se ocorreu com sucesso
     */
    function (entities) {
        var _this = this;
        return new Promise(function (resolve) {
            var /** @type {?} */ requests = [];
            entities.forEach(function (e) {
                var /** @type {?} */ provider = /** @type {?} */ (_this.injector.get(e.providerToken));
                var /** @type {?} */ state = _this.getState(e);
                state.isSyncing = true;
                requests.push(provider.getSyncData(state));
            });
            forkJoin(requests).subscribe(function (x) {
                if (x) {
                    x.forEach(function (e) {
                        if (e) {
                            _this.syncStorage.data[_this.getStateIdx(e.state.entity)] = e.state;
                        }
                    });
                    _this.syncStorage.updateStorage()
                        .then(function () {
                        x.forEach(function (e) {
                            _this.events.publish(SyncEvents.GOT_ENTITY_SYNC_DATA, e);
                        });
                        resolve(true);
                    });
                }
                else {
                    _this._syncing = false;
                    resolve(false);
                }
            }, function () {
                _this._syncing = false;
                resolve(false);
            });
        });
    };
    /**
     * Inicia o serviço que executa temporalmente a sincronização de todas as entidades,
     * caso o dispositivo esteja na rede escolhida
     * @param {?} now Se o serviço deve iniciar imediatamente ou deve aguardar o próximo cíclo
     * @return {?}
     */
    SyncProvider.prototype.startAutoWatch = /**
     * Inicia o serviço que executa temporalmente a sincronização de todas as entidades,
     * caso o dispositivo esteja na rede escolhida
     * @param {?} now Se o serviço deve iniciar imediatamente ou deve aguardar o próximo cíclo
     * @return {?}
     */
    function (now) {
        var _this = this;
        this.watchNetwork();
        this.stopAutoWatch();
        if (now) {
            this._nowToken = setTimeout(function () {
                if (_this.autoWatchEnabled) {
                    _this.syncAll();
                }
            });
            this._intervalToken = setInterval(function () {
                if (_this.autoWatchEnabled) {
                    _this.syncAll();
                }
            }, this.getNextCall());
        }
        else {
            this._scheduleToken = of({}).pipe(delay(this.getNextCall())).subscribe(function () {
                _this._intervalToken = setInterval(function () {
                    if (_this.autoWatchEnabled) {
                        _this.syncAll();
                    }
                }, _this.getNextCall());
            });
        }
    };
    /**
     * Cancela a sincronização automática.
     * @return {?}
     */
    SyncProvider.prototype.stopAutoWatch = /**
     * Cancela a sincronização automática.
     * @return {?}
     */
    function () {
        // TODO: Implementar uma espera da sincronização em andamento terminar antes de encerrar os timers ativos.
        if (this._nowToken) {
            clearInterval(this._nowToken);
            this._nowToken = undefined;
        }
        if (this._scheduleToken) {
            this._scheduleToken.unsubscribe();
            this._scheduleToken = undefined;
        }
        if (this._intervalToken) {
            clearInterval(this._intervalToken);
            this._intervalToken = undefined;
        }
    };
    /**
     * Permite que seja trocado quais os tipos de rede
     * que a sincronização automática está ativa
     * @param {...?} types
     * @return {?}
     */
    SyncProvider.prototype.changeEnabledAutoWatchNetWorks = /**
     * Permite que seja trocado quais os tipos de rede
     * que a sincronização automática está ativa
     * @param {...?} types
     * @return {?}
     */
    function () {
        var types = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            types[_i] = arguments[_i];
        }
        this.config.autoWatch.enabledOnNetwork = types;
        this.setDefaults();
    };
    /**
     * Permite que sejam trocadas as configurações de sincronização automática
     * @param {?=} newConfiguration Novas configurações
     * @return {?}
     */
    SyncProvider.prototype.changeSyncWatch = /**
     * Permite que sejam trocadas as configurações de sincronização automática
     * @param {?=} newConfiguration Novas configurações
     * @return {?}
     */
    function (newConfiguration) {
        this.config.autoWatch = newConfiguration;
        if (this.config.autoWatch) {
            this.startAutoWatch(true);
        }
    };
    /**
     * Recupera o estado de sincronização da entidade
     * @param {?} entity a entidade de sincronização
     * @return {?}
     */
    SyncProvider.prototype.getState = /**
     * Recupera o estado de sincronização da entidade
     * @param {?} entity a entidade de sincronização
     * @return {?}
     */
    function (entity) {
        var /** @type {?} */ e = this.syncStorage.data.find(function (x) { return x.entity.name === entity.name; });
        if (e === undefined) {
            var /** @type {?} */ newEntity = this.newState(entity);
            this.syncStorage.data.push(newEntity);
            e = newEntity;
        }
        return e;
    };
    /**
     * Recupera o índice do estado de sincronização da entidade no storage
     * @param {?} entity a entidade de sincronização
     * @return {?}
     */
    SyncProvider.prototype.getStateIdx = /**
     * Recupera o índice do estado de sincronização da entidade no storage
     * @param {?} entity a entidade de sincronização
     * @return {?}
     */
    function (entity) {
        var /** @type {?} */ idx = this.syncStorage.data.findIndex(function (x) { return x.entity.name === entity.name; });
        if (idx === -1) {
            var /** @type {?} */ newEntity = this.newState(entity);
            idx = this.syncStorage.data.push(newEntity) - 1;
        }
        return idx;
    };
    /**
     * Cria um novo estado de sincronização para a entidade
     * @param {?} entity a entidade de sincronização
     * @return {?}
     */
    SyncProvider.prototype.newState = /**
     * Cria um novo estado de sincronização para a entidade
     * @param {?} entity a entidade de sincronização
     * @return {?}
     */
    function (entity) {
        return { entity: entity, entityVersion: undefined, isSyncing: false };
    };
    /**
     * Calcula o tempo em ms para a próxima chamada da sincronização
     * Quando não configurado, o tempo será um minuto
     * @return {?}
     */
    SyncProvider.prototype.getNextCall = /**
     * Calcula o tempo em ms para a próxima chamada da sincronização
     * Quando não configurado, o tempo será um minuto
     * @return {?}
     */
    function () {
        if (this.config.autoWatch) {
            var /** @type {?} */ times = this.config.autoWatch.checkQuantity;
            if (times === undefined || times <= 0 || isNaN(times)) {
                times = 1;
            }
            var /** @type {?} */ type = this.config.autoWatch.checkType;
            if (type === 'days') {
                return SyncProvider.daysMs * times;
            }
            else if (type === 'hours') {
                return SyncProvider.hoursMs * times;
            }
            else if (type === 'minutes') {
                return SyncProvider.minutesMs * times;
            }
            else if (type === 'seconds') {
                return SyncProvider.secondsMs * times;
            }
            else {
                // Defaults for new types
                return SyncProvider.minutesMs * times;
            }
        }
        else {
            return SyncProvider.minutesMs;
        }
    };
    /**
     * @ignore
     * Configura os valores padrão, quando não informados
     * @return {?}
     */
    SyncProvider.prototype.setDefaults = /**
     * @ignore
     * Configura os valores padrão, quando não informados
     * @return {?}
     */
    function () {
        if (this.config.autoWatch) {
            if (this.config.autoWatch.enabledOnNetwork['push'] !== undefined) {
                var /** @type {?} */ array = /** @type {?} */ (this.config.autoWatch.enabledOnNetwork);
                if (array.length === 0) {
                    this.config.autoWatch.enabledOnNetwork = 'any';
                }
            }
        }
    };
    /**
     * @ignore
     * Assina a todos os eventos do interesse da classe
     * @return {?}
     */
    SyncProvider.prototype.subscribeEvents = /**
     * @ignore
     * Assina a todos os eventos do interesse da classe
     * @return {?}
     */
    function () {
        if (this.config.autoWatch) {
            this._currentNetworkType = this.network.type;
            if (this.config.autoWatch.auto) {
                this.startAutoWatch(false);
            }
            this.watchNetwork();
        }
    };
    /**
     * Assina o evento de trocas de rede
     * @return {?}
     */
    SyncProvider.prototype.watchNetwork = /**
     * Assina o evento de trocas de rede
     * @return {?}
     */
    function () {
        var _this = this;
        if (!this._onNetworkChange) {
            this._onNetworkChange = this.network.onchange().subscribe(function () {
                setTimeout(function () {
                    _this._currentNetworkType = _this.network.type;
                }, 3000);
            });
        }
    };
    /**
     * Constante convertendo 1 dia em milisegundos
     */
    SyncProvider.daysMs = 86400000;
    /**
     * Constante convertendo 1 hora em milisegundos
     */
    SyncProvider.hoursMs = 3600000;
    /**
     * Constante convertendo 1 minuto em milisegundos
     */
    SyncProvider.minutesMs = 60000;
    /**
     * Constante convertendo 1 segundo em milisegundos
     */
    SyncProvider.secondsMs = 1000;
    SyncProvider.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    SyncProvider.ctorParameters = function () { return [
        { type: Platform, },
        { type: Storage, },
        { type: undefined, decorators: [{ type: Inject, args: [SYNC_CONFIG,] },] },
        { type: Network, },
        { type: Events, },
        { type: Injector, },
    ]; };
    return SyncProvider;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Módulo com as dependências para sincronização mobile
 */
var SyncModule = /** @class */ (function () {
    function SyncModule() {
    }
    /**
     * @return {?}
     */
    SyncModule.forRoot = /**
     * @return {?}
     */
    function () {
        return {
            ngModule: SyncModule,
            providers: [
                SyncProvider,
                Network
            ]
        };
    };
    SyncModule.decorators = [
        { type: NgModule, args: [{},] },
    ];
    return SyncModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

var __awaiter$9 = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator$9 = (undefined && undefined.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Utilitário com funções para sincronização de dados em banco sqlite
 */
var SyncSqLiteUtil = /** @class */ (function () {
    /**
     * Creates an instance of SyncDbUtils.
     * @param cn a conexão que será utilizada para salvar os dados
     */
    function SyncSqLiteUtil(cn) {
        this.cn = cn;
    }
    /**
     * Cria um relacionamento 1-n, inserindo na entidade filha o id do pai
     *
     * @template T
     * @param {?} items os registros pai
     * @param {?} fatherTable o nome da tabela pi
     * @return {?} Os registros relacionados
     */
    SyncSqLiteUtil.prototype.updateChildrenRelation = /**
     * Cria um relacionamento 1-n, inserindo na entidade filha o id do pai
     *
     * @template T
     * @param {?} items os registros pai
     * @param {?} fatherTable o nome da tabela pi
     * @return {?} Os registros relacionados
     */
    function (items, fatherTable) {
        // FIXME: Fazer com que todos os filhos também tenha relacionamento com o pai mestre
        var /** @type {?} */ childrenFound = [];
        var /** @type {?} */ models = Object.keys(items[0]);
        for (var _i = 0, items_1 = items; _i < items_1.length; _i++) {
            var item = items_1[_i];
            var _loop_1 = function (model) {
                if (UtilsFn.isArray(item[model])) {
                    var /** @type {?} */ children = /** @type {?} */ (item[model]);
                    var /** @type {?} */ cf = childrenFound.find(function (x) { return x.table === model; });
                    if (!cf) {
                        childrenFound.push({ table: model, items: children });
                    }
                    else {
                        cf.items = cf.items.concat(children);
                    }
                    for (var _i = 0, children_1 = children; _i < children_1.length; _i++) {
                        var child = children_1[_i];
                        child[fatherTable + "Id"] = item.id;
                    }
                }
            };
            for (var _a = 0, models_1 = models; _a < models_1.length; _a++) {
                var model = models_1[_a];
                _loop_1(model);
            }
        }
        return childrenFound;
    };
    /**
     * Armazena os dados enviados, substituindo o registro anterior e criando tabelas para
     * entidades filhas(arrays)
     * @template T
     * @param {?} data A requisição de sync com os dados
     * @param {?} mainTable O nome da tabela principal
     * @param {?=} deleteBefore se deve sempre deletar antes de inserir o registro
     * @param {?=} currentTx a transação corrente, não deve ser passado sem entender o mecanismo interno
     * @return {?}
     */
    SyncSqLiteUtil.prototype.storeData = /**
     * Armazena os dados enviados, substituindo o registro anterior e criando tabelas para
     * entidades filhas(arrays)
     * @template T
     * @param {?} data A requisição de sync com os dados
     * @param {?} mainTable O nome da tabela principal
     * @param {?=} deleteBefore se deve sempre deletar antes de inserir o registro
     * @param {?=} currentTx a transação corrente, não deve ser passado sem entender o mecanismo interno
     * @return {?}
     */
    function (data, mainTable, deleteBefore, currentTx) {
        if (deleteBefore === void 0) { deleteBefore = true; }
        return __awaiter$9(this, void 0, void 0, function () {
            var _this = this;
            var children, model, error_1;
            return __generator$9(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!data.items || data.items.length === 0) {
                            return [2 /*return*/];
                        }
                        children = this.updateChildrenRelation(data.items, mainTable);
                        model = this.modelWithoutArrays(data.items[0]);
                        if (!!currentTx) return [3 /*break*/, 5];
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this.cn.transaction(function (tx) {
                                _this.checkTableStateTx(tx, mainTable, model, function (txa, rs) {
                                    for (var _i = 0, _a = data.items; _i < _a.length; _i++) {
                                        var item = _a[_i];
                                        _this.upsert(txa, mainTable, model, item, deleteBefore);
                                    }
                                    for (var _b = 0, children_2 = children; _b < children_2.length; _b++) {
                                        var child = children_2[_b];
                                        _this.storeData({
                                            items: child.items
                                        }, mainTable + "_" + child.table, deleteBefore, tx);
                                    }
                                });
                            })];
                    case 2:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _a.sent();
                        console.log(error_1);
                        return [3 /*break*/, 4];
                    case 4: return [3 /*break*/, 6];
                    case 5:
                        // Continua na transação corrente
                        this.checkTableStateTx(currentTx, mainTable, model, function (tx, rs) {
                            if (mainTable.search('_') !== -1 && data.items.length > 0) {
                                // Delete childs first
                                var /** @type {?} */ tables = mainTable.split('_');
                                tables.splice(tables.length - 1, 1);
                                var /** @type {?} */ fieldId_1 = tables.join('_') + 'Id';
                                var /** @type {?} */ fathersId_1 = [];
                                data.items.forEach(function (e) {
                                    if (!fathersId_1.find(function (x) { return x === e[fieldId_1]; })) {
                                        fathersId_1.push(e[fieldId_1]);
                                    }
                                });
                                fathersId_1.forEach(function (e) {
                                    console.log("DELETE FROM " + mainTable + " where " + fieldId_1 + " = ?", [e]);
                                    tx.executeSql("DELETE FROM " + mainTable + " where " + fieldId_1 + " = ?", [e], function (txa, res) { }, function (error) { return console.log(error); });
                                });
                            }
                            for (var _i = 0, _a = data.items; _i < _a.length; _i++) {
                                var item = _a[_i];
                                _this.upsert(currentTx, mainTable, model, item, deleteBefore);
                            }
                            for (var _b = 0, children_3 = children; _b < children_3.length; _b++) {
                                var child = children_3[_b];
                                _this.storeData({
                                    items: child.items
                                }, mainTable + "_" + child.table, deleteBefore, currentTx);
                            }
                        });
                        _a.label = 6;
                    case 6: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Retorna se a tabela existe
     * @param {?} table o nome da tabela que se deseja verificar se existe
     * @return {?}
     */
    SyncSqLiteUtil.prototype.tableExists = /**
     * Retorna se a tabela existe
     * @param {?} table o nome da tabela que se deseja verificar se existe
     * @return {?}
     */
    function (table) {
        return __awaiter$9(this, void 0, void 0, function () {
            var rs, error_2;
            return __generator$9(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.cn.executeSql('SELECT name FROM sqlite_master WHERE type=? AND name=?', ['table', table])];
                    case 1:
                        rs = _a.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        error_2 = _a.sent();
                        console.log(JSON.stringify(error_2, null, 2));
                        return [3 /*break*/, 3];
                    case 3:
                        if (rs && rs.rows.length > 0 && rs.rows.item(0).name === table) {
                            return [2 /*return*/, true];
                        }
                        return [2 /*return*/, false];
                }
            });
        });
    };
    /**
     * Cria a tabela desejada, se ela não existir
     * @param {?} table o nome da tabela
     * @param {...?} fields os campos pertencentes a tabela
     * @return {?}
     */
    SyncSqLiteUtil.prototype.createTable = /**
     * Cria a tabela desejada, se ela não existir
     * @param {?} table o nome da tabela
     * @param {...?} fields os campos pertencentes a tabela
     * @return {?}
     */
    function (table) {
        var fields = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            fields[_i - 1] = arguments[_i];
        }
        return __awaiter$9(this, void 0, void 0, function () {
            var rs_1;
            return __generator$9(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.cn.executeSql('CREATE TABLE IF NOT EXISTS ' + table + ' (' + fields.join(',') + ')')];
                    case 1:
                        _a.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        rs_1 = _a.sent();
                        console.log('success', rs_1);
                        return [3 /*break*/, 3];
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Verifica se os campos já existem na tabela e os adiciona
     * @param {?} table O nome da tabela
     * @param {...?} fields os campos que serão adicionados, se ainda não existirem na tabela
     * @return {?}
     */
    SyncSqLiteUtil.prototype.checkAndAddFields = /**
     * Verifica se os campos já existem na tabela e os adiciona
     * @param {?} table O nome da tabela
     * @param {...?} fields os campos que serão adicionados, se ainda não existirem na tabela
     * @return {?}
     */
    function (table) {
        var fields = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            fields[_i - 1] = arguments[_i];
        }
        return __awaiter$9(this, void 0, void 0, function () {
            var rs_2, _a, fields_1, field, hasField, rs_3;
            return __generator$9(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _b.trys.push([0, 2, , 9]);
                        return [4 /*yield*/, this.cn.executeSql('PRAGMA table_info(' + table + ')')];
                    case 1:
                        _b.sent();
                        return [3 /*break*/, 9];
                    case 2:
                        rs_2 = _b.sent();
                        _a = 0, fields_1 = fields;
                        _b.label = 3;
                    case 3:
                        if (!(_a < fields_1.length)) return [3 /*break*/, 8];
                        field = fields_1[_a];
                        hasField = this.checkField(rs_2, field);
                        if (!(hasField === false)) return [3 /*break*/, 7];
                        _b.label = 4;
                    case 4:
                        _b.trys.push([4, 6, , 7]);
                        return [4 /*yield*/, this.cn.executeSql("ALTER TABLE " + table + " ADD " + field)];
                    case 5:
                        _b.sent();
                        return [3 /*break*/, 7];
                    case 6:
                        rs_3 = _b.sent();
                        console.log('success', rs_3);
                        return [3 /*break*/, 7];
                    case 7:
                        _a++;
                        return [3 /*break*/, 3];
                    case 8: return [3 /*break*/, 9];
                    case 9: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Retorna um array com os nomes dos campos da entidade, removendo os que são do tipo array
     * @param {?} obj o objeto que se deseja avaliar
     * @return {?}
     */
    SyncSqLiteUtil.prototype.modelWithoutArrays = /**
     * Retorna um array com os nomes dos campos da entidade, removendo os que são do tipo array
     * @param {?} obj o objeto que se deseja avaliar
     * @return {?}
     */
    function (obj) {
        var /** @type {?} */ model = Object.keys(obj);
        for (var /** @type {?} */ i = model.length - 1; i > -1; i--) {
            var /** @type {?} */ field = model[i];
            if (UtilsFn.isArray(obj[field])) {
                model.splice(i, 1);
            }
        }
        return model;
    };
    /**
     * Executa o sql necessário para realizar o upsert de um registro, dentro de uma transação
     * @param {?} tx a transação corrente
     * @param {?} table o nome da tabela
     * @param {?} model os campos envolvidos
     * @param {?} item o item que se deseja inserir
     * @param {?} deleteBefore se deve sempre excluir o registro antes de inserir
     * @return {?}
     */
    SyncSqLiteUtil.prototype.upsert = /**
     * Executa o sql necessário para realizar o upsert de um registro, dentro de uma transação
     * @param {?} tx a transação corrente
     * @param {?} table o nome da tabela
     * @param {?} model os campos envolvidos
     * @param {?} item o item que se deseja inserir
     * @param {?} deleteBefore se deve sempre excluir o registro antes de inserir
     * @return {?}
     */
    function (tx, table, model, item, deleteBefore) {
        if (deleteBefore) {
            if (table.search('_') === -1) {
                // Pula deleção de tabelas-filha
                console.log("DELETE FROM " + table + " where id = ?", [item['id']]);
                tx.executeSql("DELETE FROM " + table + " where id = ?", [item['id']], function (txa, rsa) { }, function (error) { return console.log(error); });
            }
        }
        console.log("INSERT INTO " + table + " (" + model.join(',') + ") VALUES(" + SqlUtilsFn.objectToSqlstring(item, model) + ")");
        // tslint:disable-next-line:max-line-length
        tx.executeSql("INSERT INTO " + table + " (" + model.join(',') + ") VALUES(" + SqlUtilsFn.objectToSqlstring(item, model) + ")", undefined, function (txa, rsa) { }, function (error) { return console.log(error); });
    };
    /**
     * Na consulta de campos, verifica se o campo está presente na tabela
     * @param {?} rs o recordset com os registros
     * @param {?} field o nome do campo desejado
     * @return {?}
     */
    SyncSqLiteUtil.prototype.checkField = /**
     * Na consulta de campos, verifica se o campo está presente na tabela
     * @param {?} rs o recordset com os registros
     * @param {?} field o nome do campo desejado
     * @return {?}
     */
    function (rs, field) {
        var /** @type {?} */ hasField = false;
        for (var /** @type {?} */ i = 0; i < rs.rows.length; i++) {
            if (rs.rows.item(i).name === field) {
                hasField = true;
                break;
            }
        }
        return hasField;
    };
    /**
     * Cria ou atualização a estrutura da tabela dentro de uma transação
     * @param {?} tx a transação corrente
     * @param {?} mainTable o nome da tabela
     * @param {?} model os campos da tabela
     * @param {?} afterTx o que fazer após a transação ocorrer com sucesso
     * @return {?}
     */
    SyncSqLiteUtil.prototype.checkTableStateTx = /**
     * Cria ou atualização a estrutura da tabela dentro de uma transação
     * @param {?} tx a transação corrente
     * @param {?} mainTable o nome da tabela
     * @param {?} model os campos da tabela
     * @param {?} afterTx o que fazer após a transação ocorrer com sucesso
     * @return {?}
     */
    function (tx, mainTable, model, afterTx) {
        var _this = this;
        tx.executeSql('SELECT name FROM sqlite_master WHERE type=? AND name=?', ['table', mainTable], function (txa, rsa) {
            if (rsa && rsa.rows.length > 0 && rsa.rows.item(0).name === mainTable) {
                txa.executeSql('PRAGMA table_info(' + mainTable + ')', undefined, function (txb, rsb) {
                    var /** @type {?} */ qtd = 0;
                    var /** @type {?} */ fieldsToAdd = [];
                    for (var _i = 0, model_1 = model; _i < model_1.length; _i++) {
                        var field = model_1[_i];
                        var /** @type {?} */ hasField = _this.checkField(rsb, field);
                        if (hasField === false) {
                            qtd++;
                            fieldsToAdd.push(field);
                        }
                    }
                    if (fieldsToAdd.length === 0) {
                        afterTx(txb, rsb);
                    }
                    for (var _a = 0, fieldsToAdd_1 = fieldsToAdd; _a < fieldsToAdd_1.length; _a++) {
                        var field = fieldsToAdd_1[_a];
                        txb.executeSql("ALTER TABLE " + mainTable + " ADD " + field, undefined, function (txc, rsc) {
                            qtd--;
                            if (qtd === 0) {
                                afterTx(txc, rsc);
                            }
                        });
                    }
                });
            }
            else {
                txa.executeSql('CREATE TABLE IF NOT EXISTS ' + mainTable + ' (' + model.join(',') + ')', undefined, function (txb, rsb) {
                    afterTx(txb, rsb);
                });
            }
        });
    };
    return SyncSqLiteUtil;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Classe responsável pelo Popup utilizado no processo
 * de autenticação com Oidc no Ionic
 */
var OidcCordovaPopupWindow = /** @class */ (function () {
    /**
     * Método construtor que inicializa a classe
     * @param iab Serviço do plugin nativo ionic InAppBrowser
     * @param popupParams Parâmetros para o popup
     */
    function OidcCordovaPopupWindow(iab, popupParams) {
        this.iab = iab;
        this.popupParams = popupParams;
        this.redirectUri = this.popupParams.startUrl;
    }
    Object.defineProperty(OidcCordovaPopupWindow.prototype, "promise", {
        /** Método acessor do promise retornado pelo método Navigate */
        get: /**
         * Método acessor do promise retornado pelo método Navigate
         * @return {?}
         */
        function () {
            return this._promise;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Método responsável por realizar a navegação
     * @param navParams Parâmetros de navegação
     */
    /**
     * Método responsável por realizar a navegação
     * @param {?} navParams Parâmetros de navegação
     * @return {?}
     */
    OidcCordovaPopupWindow.prototype.navigate = /**
     * Método responsável por realizar a navegação
     * @param {?} navParams Parâmetros de navegação
     * @return {?}
     */
    function (navParams) {
        var _this = this;
        this._promise = new Promise(function (resolve, reject) {
            _this._reject = reject;
            var /** @type {?} */ errorMsg = '';
            if (!navParams || !navParams.url) {
                errorMsg = 'No url provided';
            }
            else {
                var /** @type {?} */ features = _this.popupParams.popupWindowFeatures || OidcCordovaPopupWindow.DefaultPopupFeatures;
                var /** @type {?} */ target = _this.popupParams.popupWindowTarget || OidcCordovaPopupWindow.DefaultPopupTarget;
                _this.popup = _this.iab.create(navParams.url, target, features);
                if (_this.popup) {
                    // Evento 'exit'
                    // Evento 'exit'
                    _this.popupExitSub = _this.popup.on('exit').subscribe(function (event) {
                        _this.closePopup();
                        reject(new Error(event.message));
                    });
                    // Evento 'loadstart'
                    // Evento 'loadstart'
                    _this.popupLoadStartSub = _this.popup.on('loadstart').subscribe(function (event) {
                        var /** @type {?} */ url = event['url'];
                        if (url.indexOf(_this.redirectUri) === 0) {
                            _this.closePopup();
                            resolve({ url: url });
                        }
                        else if (url.indexOf('error') !== -1) {
                            _this.closePopup();
                            reject(new Error("Load start error. URL: " + url));
                        }
                    });
                }
                else {
                    errorMsg = 'Error opening popup window';
                }
            }
            if (errorMsg !== '') {
                _this.closePopup();
                reject(new Error('Error opening popup window'));
            }
        });
        return this.promise;
    };
    /**
     * Método responsável por fechar
     * @return {?}
     */
    OidcCordovaPopupWindow.prototype.close = /**
     * Método responsável por fechar
     * @return {?}
     */
    function () {
        this.closePopup();
        this._reject();
    };
    /**
     * Método responsável por fechar o popup
     * @return {?}
     */
    OidcCordovaPopupWindow.prototype.closePopup = /**
     * Método responsável por fechar o popup
     * @return {?}
     */
    function () {
        if (this.popup) {
            this.popupExitSub.unsubscribe();
            this.popupLoadStartSub.unsubscribe();
            this.popup.close();
            this.popup = undefined;
        }
    };
    /**
     * Propriedades padrão do Popup
     */
    OidcCordovaPopupWindow.DefaultPopupFeatures = 'location=no,toolbar=no,zoom=no';
    /**
     * Destino padrão do Popup
     */
    OidcCordovaPopupWindow.DefaultPopupTarget = '_blank';
    return OidcCordovaPopupWindow;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Classe responsável pela navegação com popup utilizado no processo
 * de autenticação com Oidc no Ionic
 */
var OidcCordovaPopupNavigator = /** @class */ (function () {
    /**
     * Método construtor que inicializa a classe
     * @param iab Serviço do plugin nativo InAppBrowser
     */
    function OidcCordovaPopupNavigator(iab) {
        this.iab = iab;
    }
    /**
     * Método que prepara a janela de popup
     * @param params Parâmetros para a janela de popup
     */
    /**
     * Método que prepara a janela de popup
     * @param {?} params Parâmetros para a janela de popup
     * @return {?}
     */
    OidcCordovaPopupNavigator.prototype.prepare = /**
     * Método que prepara a janela de popup
     * @param {?} params Parâmetros para a janela de popup
     * @return {?}
     */
    function (params) {
        var /** @type {?} */ popup = new OidcCordovaPopupWindow(this.iab, params);
        return Promise.resolve(popup);
    };
    return OidcCordovaPopupNavigator;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Classe responsável pelo Iframe utilizado no processo
 * de autenticação com Oidc no Ionic
 */
var OidcCordovaIFrameNavigator = /** @class */ (function () {
    /**
     * Método construtor que inicializa a classe
     * @param iab Serviço do plugin nativo InAppBrowser
     */
    function OidcCordovaIFrameNavigator(iab) {
        this.iab = iab;
    }
    /**
     * Método que prepara a janela de popup
     * @param params Parâmetros para a janela de popup
     */
    /**
     * Método que prepara a janela de popup
     * @param {?} params Parâmetros para a janela de popup
     * @return {?}
     */
    OidcCordovaIFrameNavigator.prototype.prepare = /**
     * Método que prepara a janela de popup
     * @param {?} params Parâmetros para a janela de popup
     * @return {?}
     */
    function (params) {
        params['popupWindowFeatures'] = 'hidden=yes';
        params['isIframe'] = true;
        var /** @type {?} */ popup = new OidcCordovaPopupWindow(this.iab, params);
        return Promise.resolve(popup);
    };
    return OidcCordovaIFrameNavigator;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

export { FndModule, MockModule, AuthModule, AuthService, DateTimeModule, GlobalService, InterceptorModule, DefaultHttpInterceptorService, IntlLiteralsLoaderService, IntlModule, I18NPipe, IntlService, createTranslateLoader, ShowMessageModule, ShowMessageService, ShowLoadingService, ShowLoadingModule, CameraUtilModule, CameraUtilEvents, CameraUtilProvider, FileUtilModule, FileUtilProvider, GeolocationUtilModule, GeolocationUtilProvider, HttpOauth2UtilModule, HttpOauth2UtilProvider, ImagePickerUtilEvents, ImagePickerUtilModule, ImagePickerUtilProvider, QRScannerUtilModule, QrScannerUtilProvider, SpeechRecognitionUtilModule, SpeechRecognitionUtilProvider, StorageUtil, SqLiteUtilBaseProvider, SqLiteUtilProvider, SqLiteUtilModule, SyncHandler, SyncEvents, SyncProvider, SYNC_CONFIG, SyncModule, SyncSqLiteUtil, OidcCordovaPopupWindow, OidcCordovaPopupNavigator, OidcCordovaIFrameNavigator };
