/**
 * Esta classe simula a classe Platform e é utilizada
 * para os testes unitários automatizados
 * ```typescript
 * // Utilizar como provider
 * providers: [
 *   { provide: Platform, useClass: PlatformMock},
 *   ...
 * ]
 * ```
 */
export declare class PlatformMock {
    /**
     * Método que simula a ação de preparado da plataforma
     */
    ready(): Promise<string>;
    /**
     * Método que simula o retorno dos parâmetros de query de URL
     */
    getQueryParam(): boolean;
    /**
     * Método que simula o registro da ação para o botão 'Voltar'
     * @param fn
     * @param priority
     */
    registerBackButtonAction(fn: Function, priority?: number): Function;
    /**
     * Método que simula a verificação de foco
     * @param ele
     */
    hasFocus(ele: HTMLElement): boolean;
    /**
     * Método que simula o retorno do 'document'
     */
    doc(): HTMLDocument;
    /**
     * Método que simula o retorno do 'is'
     */
    is(): boolean;
    /**
     * Método que simula o retorno do estilo de um elemento
     * @param container
     */
    getElementComputedStyle(container: any): any;
    /**
     * Método que simula o evento de redimensionamento de tela
     * @param callback
     */
    onResize(callback: any): any;
    /**
     * Método que simula o registro de evento de escuta
     * @param ele
     * @param eventName
     * @param callback
     */
    registerListener(ele: any, eventName: string, callback: any): Function;
    /**
     * Método que simula o retorno do método 'win'
     */
    win(): Window;
    /**
     * Método que simula o retorno do método 'raf'
     * @param callback
     */
    raf(callback: any): number;
    /**
     * Método que simula a definição de tempo de espera
     * @param callback
     * @param timer
     */
    timeout(callback: any, timer: number): any;
    /**
     * Método que simula o cancelamento do tempo de espera
     * @param id
     */
    cancelTimeout(id: any): void;
    /**
     * Método que simula o retorno do elemento ativo
     */
    getActiveElement(): any;
}
