import { File, RemoveResult } from '@ionic-native/file';
import { Platform } from 'ionic-angular';
import { IFileUtil } from './file-util.interface';
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de arquivo
 * com métodos que facilitam a manipulação de arquivos no aplicativo
 */
export declare class FileUtilProvider implements IFileUtil {
    private file;
    /**
     * @ignore
     * Indica se a plataforma é cordova
     */
    private isCordova;
    /**
     * @ignore
     * Método construtor responsável por inicializar o serviço
     * @param platform Serviço relacionado a plataforma
     * @param file Serviço do plugin nativo de arquivo
     * @param events Serviço de eventos do Ionic
     */
    constructor(platform: Platform, file: File);
    /**
     * Método responsável por remover um arquivo
     * @param uri URI do arquivo que deve ser removido
     * @return Objeto que representa o resultado da remoção do arquivo
     */
    removeFile(uri: string): Promise<RemoveResult>;
    /**
     * Método responsável por transferir os arquivos para o storage do aplicativo
     * @param uris URIs dos arquivos que devem ser copiados
     * @return Lista de URIs dos arquivos copiados para o storage do aplicativo
     */
    fileCacheToAppStorage(...uris: string[]): Promise<string[]>;
    /**
     * @ignore
     * Método responsável por recuperar o nome do arquivo de um URI
     * @param uri URI do arquivo
     */
    private getFilenameFromUri(uri);
    /**
     * @ignore
     * Método responsável por recuperar o path do arquivo de um URI
     * @param uri URI do arquivo
     */
    private getPathFromUri(uri);
}
