import { GlobalService } from './../../services/global/global.service';
import { Platform } from 'ionic-angular';
import { IGeolocationUtil } from './geolocation-util.interface';
import { Geolocation, GeolocationOptions, Geoposition } from '@ionic-native/geolocation';
import { IGeoPosition } from './geo-position.interface';
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de geolocalização
 * com métodos que facilitam a solicitação de permissão e identificação de geolocalização
 */
export declare class GeolocationUtilProvider implements IGeolocationUtil {
    private geolocation;
    private global;
    /**
     * @ignore
     * Indica se a plataforma é cordova
     */
    private isCordova;
    /**
     * @ignore
     * Método responsável por construir o serviço
     * @param platform Serviço relacionado a plataforma
     * @param geolocation Serviço do plugin nativo de geolocalização
     * @param global Serviço global da aplicação
     */
    constructor(platform: Platform, geolocation: Geolocation, global: GlobalService);
    /**
     * Método responsável por obter a posição corrente do usuário
     * @param {GeolocationOptions} options Opções para recuperação da geolocalização
     * @return Posição
     */
    getCurrentPosition(options: GeolocationOptions): Promise<Geoposition>;
    /**
     * Método responsável por calcular a distância entre dois pontos através de suas latitudes e longitudes
     * @param point1 Coordenadas do primeiro ponto
     * @param point2 Coordenadas do segundo ponto
     */
    calculatePointsKMDistance(point1: IGeoPosition, point2: IGeoPosition): number;
    /**
     * Converte os graus decimais em radianos
     * @param degrees Graus decimais
     */
    degreesToRad(degrees: number): number;
    /**
     * Valida os valores do ponto recebido e retorna o resultado
     * @param point Ponto contendo latitude e longitude
     */
    validGeoPositionPoint(point: IGeoPosition): boolean;
}
