import { IHttpUtilResult } from './http-util-result.interface';
import { AuthBaseService } from 'base-foundation';
import { Platform } from 'ionic-angular';
import { IHttpUtil } from './http-util.interface';
import { HTTP } from '@ionic-native/http';
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de requisição http
 * nativa do dispositivo com métodos que facilitam a utilização do plugin
 */
export declare class HttpOauth2UtilProvider implements IHttpUtil {
    private auth;
    private http;
    /**
     * @ignore
     * Indica se a plataforma é cordova
     */
    private isCordova;
    /**
     * @ignore
     * Método construtor responsável por inicializar o serviço
     * @param platform Serviço relacionado a plataforma
     * @param file Serviço do plugin nativo de arquivo
     * @param events Serviço de eventos do Ionic
     */
    constructor(platform: Platform, auth: AuthBaseService, http: HTTP);
    /**
     * Método responsável por realizar o upload de um arquivo
     * @template T Objeto de resposta do servidor quando sucesso
     * @param url URL do Endpoint de Upload
     * @param filePath URI do arquivo no dispositivo
     * @param name Nome do arquivo
     * @param body Informações no corpo da mensagem
     * @param headers Informações de cabeçalho da mensagem
     * @return {IHttpUtilResult} Resultado da requisição
     */
    uploadFile<T>(url: string, filePath: string, name?: string, body?: any, headers?: any): Promise<IHttpUtilResult<T>>;
    /**
     * Método que permite a alteração do modo de verificação do certificado SSL
     * @param mode Modo de verificação do certificado SSL
     */
    setSSLCertMode(mode: 'default' | 'nocheck' | 'pinned'): Promise<void>;
}
