import { IHttpUtilResult } from './http-util-result.interface';
/**
 * Interface que define as propriedades e comportamentos do
 * utilitário de requisição Http
 */
export interface IHttpUtil {
    /**
     * Método responsável por realizar o upload de um arquivo
     * @template T Objeto de resposta do servidor quando sucesso
     * @param url URL do Endpoint de Upload
     * @param filePath URI do arquivo no dispositivo
     * @param name Nome do arquivo
     * @param body Informações no corpo da mensagem
     * @param headers Informações de cabeçalho da mensagem
     * @return {IHttpUtilResult} Resultado da requisição
     */
    uploadFile<T>(url: string, filePath: string, name?: string, body?: any, headers?: any): Promise<IHttpUtilResult<T>>;
    /**
     * Método que permite a alteração do modo de verificação do certificado SSL
     * @param mode Modo de verificação do certificado SSL
     */
    setSSLCertMode(mode: 'default' | 'nocheck' | 'pinned'): any;
}
