import { GlobalService } from './../../services/global/global.service';
import { IImagePickerUtil } from './image-picker-util.interface';
import { Platform } from 'ionic-angular';
import { ImagePicker, ImagePickerOptions } from '@ionic-native/image-picker';
import { Events } from 'ionic-angular';
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de seleção
 * de imagens do dispositivo com métodos que facilitam a solicitação de permissão,
 * apresentação e seleção de imagens
 */
export declare class ImagePickerUtilProvider implements IImagePickerUtil {
    private imagePicker;
    private global;
    private events;
    /**
     * @ignore
     * Indica se a plataforma é cordova
     */
    private isCordova;
    /**
     * @ignore
     * Opções padrões para a galeria
     */
    private defaultOptions;
    /**
     * @ignore
     * Método responsável por construir o serviço
     * @param platform Serviço relacionado a plataforma
     * @param imagePicker Serviço do plugin nativo de galeria
     * @param global Serviço global da aplicação
     */
    constructor(platform: Platform, imagePicker: ImagePicker, global: GlobalService, events: Events);
    /**
     * Método responsável por abrir o seletor de imagens e retornar o caminho dos arquivos salvos no cache
     * Este método publica o evento ImagePickerUtilEvents.CACHE_FILE_CREATED quando um ou mais arquivos são selecionados
     * e criados em cache (Mais informações sobre eventos {@link https://ionicframework.com/docs/api/util/Events/|Ionic Events})
     * @param opt Customização das opções enviadas ao seletor de images
     * @return Lista de imagens escolhidas
     */
    chooseImage(opt?: ImagePickerOptions): Promise<any>;
    /**
     * Método responsável por requisitar permissão para acesso a galeria
     * @return Indicação se possui ou não permissão
     */
    requestReadPermission(): Promise<boolean>;
}
