import { GlobalService } from './../../services/global/global.service';
import { IQrScannerUtil } from './qr-scanner-util.interface';
import { QRScanner, QRScannerStatus } from '@ionic-native/qr-scanner';
import { Subject } from 'rxjs';
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de leitura de QR Code
 * com métodos que facilitam a solicitação de permissão e leitura do código
 */
export declare class QrScannerUtilProvider implements IQrScannerUtil {
    private qrScanner;
    private global;
    /**
     * Propriedade que possui informações sobre o QR Code Scanner após ele estar
     * preparado, indicando assim se é possível ligar o flash ou trocar de camera,
     * por exemplo.
     */
    qrScannerLastStatus: Subject<QRScannerStatus>;
    /**
     * @ignore
     * Assinatura do evento de leitura do QR-Code
     */
    private scanSub;
    /**
     * @ignore
     * Método responsável por construir o serviço
     * @param qrScanner Serviço do plugin nativo de leitura do QR-Code
     * @param global Serviço global da aplicação
     */
    constructor(qrScanner: QRScanner, global: GlobalService);
    /**
     * Método responsável por preparar o plugin para leitura do QR-Code
     * @return Indicação da preparação
     */
    prepare(): Promise<boolean>;
    /**
     * Método responsável por apresentar o leitor do QR-Code
     * @return String com o conteúdo lido
     */
    show(): Promise<string>;
    /**
     * Método responsável por esconder a camera da leitura do QR-Code
     */
    hide(): void;
    /**
     * Método responsável por ligar ou desligar o flash
     */
    flash(): void;
    /**
     * Método responsável por alternar entre a camera frontal e a camera traseira
     */
    changeCamera(): void;
    /**
     * Método responsável por destruir o plugin nativo desabilitando a leitura do QR-Code até
     * que o plugin seja novamente preparado
     */
    destroy(): void;
}
