import { GlobalService } from './../../services/global/global.service';
import { ISpeechRecognitionUtil } from './speech-recognition-util.interface';
import { Platform } from 'ionic-angular';
import { SpeechRecognition } from '@ionic-native/speech-recognition';
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de reconhecimento de voz
 * com métodos que facilitam a solicitação de permissão e reconhecimento
 */
export declare class SpeechRecognitionUtilProvider implements ISpeechRecognitionUtil {
    private speechRecognition;
    private global;
    /**
     * @ignore
     * Indica se a plataforma é cordova
     */
    private isCordova;
    /**
     * @ignore
     * Opções do reconhecimento
     */
    private options;
    /**
     * @ignore
     * Método responsável por construir o serviço
     * @param platform Serviço relacionado a plataforma
     * @param speechRecognition Serviço do plugin nativo de reconhecimento de voz
     * @param global Serviço global da aplicação
     */
    constructor(platform: Platform, speechRecognition: SpeechRecognition, global: GlobalService);
    /**
     * Método responsável por habilitar o reconheicimento de voz
     * @return Lista com os textos reconhecidos (geralmente o primeiro item é o mais provável)
     */
    listening(): Promise<string[]>;
    /**
     * @ignore
     * Método responsável por verificar se há permissão e se não houver, solicitar
     * @return Indicação se há ou não permissão
     */
    private verifyPermission();
    /**
     * @ignore
     * Método responsável por solicitar permissão de voz
     * @return Indicação se a permissão foi ou não concedida
     */
    private requestPermission();
    /**
     * @ignore
     * Método para apresentar mensagens
     * @param literalMessage Literal da mensagem
     */
    private showToast(literalMessage);
}
