import { SQLiteObject, DbTransaction } from '@ionic-native/sqlite';
import { ISyncItemDto } from 'base-foundation';
import { IChildren } from './children.interface';
/**
 * Utilitário com funções para sincronização de dados em banco sqlite
 */
export declare class SyncSqLiteUtil {
    private cn;
    /**
     * Creates an instance of SyncDbUtils.
     * @param cn a conexão que será utilizada para salvar os dados
     */
    constructor(cn: SQLiteObject);
    /**
     * Cria um relacionamento 1-n, inserindo na entidade filha o id do pai
     *
     * @template T
     * @param items os registros pai
     * @param fatherTable o nome da tabela pi
     * @returns Os registros relacionados
     */
    updateChildrenRelation<T extends ISyncItemDto>(items: T[], fatherTable: string): Array<IChildren<T>>;
    /**
     * Armazena os dados enviados, substituindo o registro anterior e criando tabelas para
     * entidades filhas(arrays)
     * @param data A requisição de sync com os dados
     * @param mainTable O nome da tabela principal
     * @param deleteBefore se deve sempre deletar antes de inserir o registro
     * @param currentTx a transação corrente, não deve ser passado sem entender o mecanismo interno
     */
    storeData<T extends ISyncItemDto>(data: {
        items: T[];
    }, mainTable: string, deleteBefore?: boolean, currentTx?: DbTransaction): Promise<void>;
    /**
     * Retorna se a tabela existe
     * @param table o nome da tabela que se deseja verificar se existe
     */
    tableExists(table: string): Promise<boolean>;
    /**
     * Cria a tabela desejada, se ela não existir
     * @param table o nome da tabela
     * @param fields os campos pertencentes a tabela
     */
    createTable(table: string, ...fields: string[]): Promise<void>;
    /**
     * Verifica se os campos já existem na tabela e os adiciona
     * @param table O nome da tabela
     * @param fields os campos que serão adicionados, se ainda não existirem na tabela
     */
    checkAndAddFields(table: string, ...fields: string[]): Promise<void>;
    /**
     * Retorna um array com os nomes dos campos da entidade, removendo os que são do tipo array
     * @param obj o objeto que se deseja avaliar
     */
    private modelWithoutArrays(obj);
    /**
     * Executa o sql necessário para realizar o upsert de um registro, dentro de uma transação
     * @param tx a transação corrente
     * @param table o nome da tabela
     * @param model os campos envolvidos
     * @param item o item que se deseja inserir
     * @param deleteBefore se deve sempre excluir o registro antes de inserir
     */
    private upsert(tx, table, model, item, deleteBefore);
    /**
     * Na consulta de campos, verifica se o campo está presente na tabela
     * @param rs o recordset com os registros
     * @param field o nome do campo desejado
     */
    private checkField(rs, field);
    /**
     * Cria ou atualização a estrutura da tabela dentro de uma transação
     * @param tx a transação corrente
     * @param mainTable o nome da tabela
     * @param model os campos da tabela
     * @param afterTx o que fazer após a transação ocorrer com sucesso
     */
    private checkTableStateTx(tx, mainTable, model, afterTx);
}
